@@ -1,7 +1,7 @@
 from dataclasses import asdict, fields
 from enum import Enum, EnumMeta
 import inspect
-from typing import Any, Dict, Iterable, Tuple, TypeVar
+from typing import Any, Dict, Iterable, Tuple, TypeVar, Union
 
 import yaml
 
@@ -25,7 +25,9 @@ class ModelEnumType(EnumMeta):
                 "Must be a Dict[ModelEnum, str] documenting all enum values"
             )
 
-        missing = {v.value for v in cls} - set(cls.__docstrings__.keys())
+        missing = {v.value for v in cls if not v.name.startswith("_")} - set(
+            cls.__docstrings__.keys()
+        )
         if missing:
             raise ValueError(
                 f"ModelEnum '{cls.__name__}.__docstrings__' is missing docstrings for values: {sorted(missing)}"
@@ -41,6 +43,16 @@ class ModelEnum(str, Enum, metaclass=ModelEnumType):
     def __str__(self):
         return self.name
 
+    @classmethod
+    def validate(cls, value: Union[str, "ModelEnum"]) -> "ModelEnum":
+        allowed_values = [str(s) for s in cls.__members__ if not str(s).startswith("_")]
+        if value.upper() in allowed_values:
+            return cls(value.upper())
+        else:
+            raise ValueError(
+                f"'{value.upper()}' is not a valid {cls.__name__}. Allowed values are {allowed_values}"
+            )
+
 
 class ModelBaseType(type):
     pass
@@ -68,6 +80,9 @@ class ModelBase(metaclass=ModelBaseType):
         If the validator has a return type annotation, the field will be updated to the
         returned value.
         """
+        if getattr(self, "__ignore_validation__", False):
+            return
+
         for field in fields(self):
             if not field.metadata.get("docstring", None):
                 raise ValueError(
