@@ -4,6 +4,7 @@ from anyscale._private.anyscale_client import (
     AnyscaleClient,
     AnyscaleClientInterface,
 )
+from anyscale._private.sdk.timer import RealTimer, Timer
 from anyscale.cli_logger import BlockLogger
 
 
@@ -15,9 +16,11 @@ class BaseSDK:
         *,
         logger: Optional[BlockLogger] = None,
         client: Optional[AnyscaleClientInterface] = None,
+        timer: Optional[Timer] = None,
     ):
         self._logger = logger or BlockLogger()
         self._client = client or AnyscaleClient()
+        self._timer = timer or RealTimer()
 
     @property
     def logger(self) -> BlockLogger:
@@ -26,3 +29,7 @@ class BaseSDK:
     @property
     def client(self) -> AnyscaleClientInterface:
         return self._client
+
+    @property
+    def timer(self) -> Timer:
+        return self._timer
