@@ -40,6 +40,7 @@ class AuthenticationBlock:
 
     def __init__(
         self,
+        *,
         cli_token: Optional[str] = None,
         host: Optional[str] = None,
         use_asyncio: bool = False,
@@ -80,7 +81,7 @@ class AuthenticationBlock:
         if validate_credentials:
             self._validate_credentials_format(self.credentials)
 
-        self.api_client = self._instantiate_api_client(raise_structured_exception)
+        self.api_client = self._instantiate_api_client(raise_structured_exception,)
         self.anyscale_api_client = self._instantiate_anyscale_client(
             raise_structured_exception, use_asyncio=use_asyncio
         )
@@ -120,6 +121,7 @@ class AuthenticationBlock:
             cookie=cookie,
             raise_structured_exception=raise_structured_exception,
         )
+
         configure_open_api_client_headers(api_client, "CLI")
         configure_tcp_keepalive(api_client)
         api_instance = openapi_client.DefaultApi(api_client)
@@ -155,6 +157,7 @@ class AuthenticationBlock:
                 raise_structured_exception=raise_structured_exception,
             )
         )
+
         configure_open_api_client_headers(api_client, "CLI")
         configure_tcp_keepalive(api_client)
         api_instance = anyscale_client.DefaultApi(api_client)
