@@ -3,7 +3,7 @@ from contextlib import contextmanager
 import os
 import sys
 import time
-from typing import Any, List, Optional
+from typing import List, Optional
 
 from botocore.exceptions import ClientError
 import click
@@ -49,7 +49,7 @@ class BlockLogger:
         self,
         log_output: bool = True,
         t0: float = _process_start_time,
-        spinner_manager: Any = None,
+        spinner_manager: Optional[Halo] = None,
     ) -> None:
         self.t0 = t0
         # Flag to disable all terminal output from CLILogger (useful for SDK)
@@ -226,24 +226,19 @@ class BlockLogger:
         finally:
             self.indent_level -= 1
 
-
-class LogsLogger(BlockLogger):
-    """ This logger is used to print customer logs to STDOUT with no decoration
-    """
-
-    def log(self, msg: str):
-        print(msg)
-
-    def is_interactive_cli_enabled(self) -> bool:
-        """Check if shell is interactive
-        """
-
-        default_env_var = "1" if sys.stderr.isatty() else "0"
-        return os.environ.get("ANYSCALE_CLI_INTERACTIVE_UX", default_env_var) == "1"
-
     @contextmanager
     def spinner(self, msg: str):
-        """ Create a spinner with the starting text.
+        """DEPRECATED:
+
+        See https://github.com/manrajgrover/halo/issues/179.
+        This `Halo` spinner is not compatible with newer Python notebook versions.
+        (iPython and Jupyter)
+
+        Use `rich.spinner.Spinner` or `rich.progress.Progress` instead.
+
+        ---
+        Create a spinner with the starting text.
+
         To update the text next to the spinner, set `spinner.text = "new_text"`
         The spinner will be stopped when the context is exited.
         Spinner is only enabled for interactive shell
@@ -256,6 +251,21 @@ class LogsLogger(BlockLogger):
         ) as spinner:
             yield spinner
 
+    def is_interactive_cli_enabled(self) -> bool:
+        """Check if shell is interactive
+        """
+
+        default_env_var = "1" if sys.stderr.isatty() else "0"
+        return os.environ.get("ANYSCALE_CLI_INTERACTIVE_UX", default_env_var) == "1"
+
+
+class LogsLogger(BlockLogger):
+    """ This logger is used to print customer logs to STDOUT with no decoration
+    """
+
+    def log(self, msg: str):
+        print(msg)
+
 
 class StringLogger(LogsLogger):
     """ This logger is used to print customer logs to a string with no decoration
@@ -279,7 +289,7 @@ class CloudSetupLogger(LogsLogger):
         self,
         log_output: bool = True,
         t0: float = _process_start_time,
-        spinner_manager: Any = None,
+        spinner_manager: Optional[Halo] = None,
     ):
         self.cloud_resource_errors: List[CloudAnalyticsEventCloudProviderError] = []
         super().__init__(log_output, t0, spinner_manager)
