@@ -5,6 +5,7 @@ import click
 
 from anyscale.cli_logger import BlockLogger
 from anyscale.client.openapi_client.models import ClusterManagementStackVersions
+from anyscale.client.openapi_client.models.compute_stack import ComputeStack
 from anyscale.commands import cloud_commands_util
 from anyscale.commands.util import OptionPromptNull
 from anyscale.controllers.cloud_controller import CloudController
@@ -78,6 +79,15 @@ def cloud_set_default(
     )
 
 
+def default_region(provider: str) -> str:
+    if provider == "aws":
+        return "us-west-2"
+    elif provider == "gcp":
+        return "us-west1"
+    else:
+        return "default"
+
+
 @cloud_cli.command(name="setup", help="Set up a cloud provider.")
 @click.option(
     "--provider",
@@ -93,7 +103,7 @@ def cloud_set_default(
     required=True,
     prompt="Region",
     default_option="provider",
-    default=lambda p: "us-west-2" if p == "aws" else "us-west1",
+    default=default_region,
     show_default=True,
 )
 @click.option("--name", "-n", help="Name of the cloud.", required=True, prompt="Name")
@@ -218,7 +228,10 @@ def cloud_config_group() -> None:
 
 
 @cloud_cli.command(
-    name="update", help=("Update managed cloud."),
+    name="update",
+    help=(
+        "Update a managed cloud to the latest configuration. Only applicable for anyscale managed clouds."
+    ),
 )
 @click.argument("cloud-name", required=False)
 @click.option(
@@ -310,6 +323,74 @@ def cloud_config_get(
     )
 
 
+@cloud_config_group.command(
+    "update", help="Update the current configuration for a cloud."
+)
+@click.argument("cloud-name", required=False)
+@click.option("--name", "-n", help="Update configuration of cloud by name.", type=str)
+@click.option(
+    "--cloud-id",
+    "--id",
+    help="Cloud id to get details about. Alternative to cloud name.",
+    required=False,
+)
+@click.option(
+    "--enable-log-ingestion/--disable-log-ingestion",
+    default=None,
+    help=(
+        "If --enable-log-ingestion is specified for a cloud, it will enable the log "
+        "viewing and querying UI features for the clusters on this cloud. This will "
+        "enable easier debugging. The logs produced by the clusters will "
+        "be sent from the data plane to the control plane. Anyscale does not share "
+        "this data with any third party or use it for any purpose other than serving "
+        "the log UI for the customer. The log will be stored at most 30 days."
+        "Please note by disable this feature again, Anyscale doesn't "
+        "delete the logs that have already been ingested. Your clusters may incur "
+        "extra data transfer cost from the cloud provider by enabling this feature."
+    ),
+)
+def cloud_config_update(
+    cloud_name: Optional[str],
+    name: Optional[str],
+    cloud_id: Optional[str],
+    enable_log_ingestion: Optional[bool],
+) -> None:
+    if any([enable_log_ingestion is not None]):
+        if enable_log_ingestion is True:
+            consent_message = click.prompt(
+                "--enable-log-ingestion is specified. Please note the logs produced by "
+                "your cluster will be ingested into Anyscale's service in region "
+                "us-west-2. Your clusters may incur extra data transfer cost from the "
+                "cloud provider. If you are sure you want to enable this feature, "
+                'please type "consent"',
+                type=str,
+            )
+            if consent_message != "consent":
+                raise click.ClickException(
+                    'You must type "consent" to enable log ingestion.'
+                )
+        if enable_log_ingestion is False:
+            confirm_response = click.confirm(
+                "--disable-log-ingestion is specified. Please note the logs that's "
+                "already ingested will not be deleted. Existing clusters will not stop"
+                "the log ingestion until you restart them. Logs are automatically "
+                "deleted after 30 days from the time of ingestion. Are you sure you "
+                "want to disable log ingestion?"
+            )
+            if not confirm_response:
+                raise click.ClickException("You must confirm to disable log ingestion.")
+
+        CloudController().update_cloud_config(
+            cloud_name=cloud_name or name,
+            cloud_id=cloud_id,
+            enable_log_ingestion=enable_log_ingestion,
+        )
+    else:
+        raise click.ClickException(
+            "Please provide at least one of the following arguments: --enable-log-ingestion, --disable-log-ingestion."
+        )
+
+
 @cloud_cli.command(
     name="register", help="Register an anyscale cloud with your own resources."
 )
@@ -317,7 +398,7 @@ def cloud_config_get(
     "--provider",
     help="The cloud provider type.",
     required=True,
-    type=click.Choice(["aws", "gcp"], case_sensitive=False),
+    type=click.Choice(["aws", "gcp", "generic"], case_sensitive=False),
 )
 @click.option(
     "--region",
@@ -325,9 +406,18 @@ def cloud_config_get(
     help="Region to set up the credentials in.",
     required=True,
     default_option="provider",
-    default=lambda p: "us-west-2" if p == "aws" else "us-west1",
+    default=default_region,
     show_default=True,
 )
+@click.option(
+    "--compute-stack",
+    help="The compute stack type (VM or K8S).",
+    required=False,
+    type=click.Choice([ComputeStack.VM, ComputeStack.K8S], case_sensitive=False),
+    default=ComputeStack.VM,
+    # TODO (shomilj): Unhide this option when full support for Kubernetes has been rolled out.
+    hidden=True,
+)
 @click.option(
     "--name", "-n", help="Name of the cloud.", required=True,
 )
@@ -364,6 +454,12 @@ def cloud_config_get(
 @click.option(
     "--s3-bucket-id", help="S3 bucket ID.", required=False, type=str,
 )
+@click.option(
+    "--external-id",
+    help="The trust policy external ID for the cross account IAM role.",
+    required=False,
+    type=str,
+)
 @click.option(
     "--memorydb-cluster-id", help="Memorydb cluster ID", required=False, type=str,
 )
@@ -420,7 +516,26 @@ def cloud_config_get(
 )
 @click.option(
     "--cloud-storage-bucket-name",
-    help="Cloud storage bucket name for GCP clouds",
+    help="A fully qualified storage bucket name for cloud storage, e.g. s3://bucket-name, gs://bucket-name, or azure://bucket-name.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--cloud-storage-bucket-endpoint",
+    help="An endpoint for cloud storage, e.g. used to override the default cloud storage scheme's endpoint (e.g. for S3, this would be passed to the AWS_ENDPOINT_URL environment variable).",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--nfs-mount-target",
+    help="A comma-separated value representing a (zone, mount target) tuple, e.g. us-west-2a,1.2.3.4 (may be provided multiple times, one for each zone). If only one value is provided (e.g. 1.2.3.4), then that value will be used for all zones.",
+    required=False,
+    type=str,
+    multiple=True,
+)
+@click.option(
+    "--nfs-mount-path",
+    help="The path of the NFS server to mount from (e.g. <nfs-target-address>/<nfs-path> will be mounted).",
     required=False,
     type=str,
 )
@@ -436,6 +551,30 @@ def cloud_config_get(
     required=False,
     type=str,
 )
+@click.option(
+    "--kubernetes-namespaces",
+    help="On the Kubernetes compute stack, a comma-separated list of namespace(s) that can be used for running workloads. The first namespace in the list will be used as the default.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--kubernetes-ingress-external-address",
+    help="On the Kubernetes compute stack, the address of the ingress used for serving traffic for Anyscale services.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--kubernetes-zones",
+    help="On the Kubernetes compute stack, a comma-separated list of zones to launch pods in.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--kubernetes-dataplane-identity",
+    help="On the Kubernetes compute stack, the IAM identity for the Kubernetes Manager deployment. IN AWS EKS, this is the ARN of the IAM role associated with the Kubernetes Manager deployment. For GCP GKE, this is the service account email associated with the Kubernetes Manager deployment.",
+    required=False,
+    type=str,
+)
 @click.option(
     "--private-network", help="Use private network.", is_flag=True, default=False,
 )
@@ -471,6 +610,7 @@ def cloud_config_get(
 def register_cloud(  # noqa: PLR0913, PLR0912
     provider: str,
     region: str,
+    compute_stack: ComputeStack,
     name: str,
     vpc_id: str,
     subnet_ids: str,
@@ -479,6 +619,7 @@ def register_cloud(  # noqa: PLR0913, PLR0912
     instance_iam_role_id: str,
     security_group_ids: str,
     s3_bucket_id: str,
+    external_id: Optional[str],
     memorydb_cluster_id: str,
     project_id: str,
     vpc_name: str,
@@ -490,8 +631,15 @@ def register_cloud(  # noqa: PLR0913, PLR0912
     provider_name: str,
     firewall_policy_names: str,
     cloud_storage_bucket_name: str,
+    cloud_storage_bucket_endpoint: Optional[str],
+    nfs_mount_target: List[str],
+    nfs_mount_path: str,
     memorystore_instance_name: str,
     host_project_id: Optional[str],
+    kubernetes_namespaces: Optional[str],
+    kubernetes_ingress_external_address: Optional[str],
+    kubernetes_zones: Optional[str],
+    kubernetes_dataplane_identity: Optional[str],
     functional_verify: Optional[str],
     private_network: bool,
     yes: bool,
@@ -500,18 +648,35 @@ def register_cloud(  # noqa: PLR0913, PLR0912
 ) -> None:
     missing_args: List[str] = []
     if provider == "aws":
-        # Check for missing required arguments for AWS clouds
-        # The only optional argument is `memorydb_cluster_id`
+        # Check for missing required arguments for AWS clouds,
+        # based on the compute stack (not all args are required
+        # on all compute stacks).
         for resource in [
-            (vpc_id, "--vpc-id"),
-            (subnet_ids, "--subnet-ids"),
-            (efs_id, "--efs-id"),
-            (anyscale_iam_role_id, "--anyscale-iam-role-id"),
-            (instance_iam_role_id, "--instance-iam-role-id"),
-            (security_group_ids, "--security-group-ids"),
-            (s3_bucket_id, "--s3-bucket-id"),
+            (vpc_id, "--vpc-id", (ComputeStack.VM)),
+            (subnet_ids, "--subnet-ids", (ComputeStack.VM)),
+            (efs_id, "--efs-id", (ComputeStack.VM)),
+            (
+                anyscale_iam_role_id,
+                "--anyscale-iam-role-id",
+                (ComputeStack.VM, ComputeStack.K8S),
+            ),
+            (instance_iam_role_id, "--instance-iam-role-id", (ComputeStack.VM)),
+            (security_group_ids, "--security-group-ids", (ComputeStack.VM)),
+            (s3_bucket_id, "--s3-bucket-id", (ComputeStack.VM, ComputeStack.K8S)),
+            (kubernetes_namespaces, "--kubernetes-namespaces", (ComputeStack.K8S)),
+            (
+                kubernetes_ingress_external_address,
+                "--kubernetes-ingress-external-address",
+                (ComputeStack.K8S),
+            ),
+            (kubernetes_zones, "--kubernetes-zones", (ComputeStack.K8S)),
+            (
+                kubernetes_dataplane_identity,
+                "--kubernetes-dataplane-identity",
+                (ComputeStack.K8S),
+            ),
         ]:
-            if resource[0] is None:
+            if compute_stack in resource[2] and resource[0] is None:
                 missing_args.append(resource[1])
 
         if len(missing_args) > 0:
@@ -519,39 +684,73 @@ def register_cloud(  # noqa: PLR0913, PLR0912
 
         CloudController().register_aws_cloud(
             region=region,
+            compute_stack=compute_stack,
             name=name,
             vpc_id=vpc_id,
-            subnet_ids=subnet_ids.split(","),
+            subnet_ids=subnet_ids.split(",") if subnet_ids else [],
             efs_id=efs_id,
             anyscale_iam_role_id=anyscale_iam_role_id,
             instance_iam_role_id=instance_iam_role_id,
-            security_group_ids=security_group_ids.split(","),
+            security_group_ids=security_group_ids.split(",")
+            if security_group_ids
+            else [],
             s3_bucket_id=s3_bucket_id,
             memorydb_cluster_id=memorydb_cluster_id,
             functional_verify=functional_verify,
+            kubernetes_namespaces=kubernetes_namespaces.split(",")
+            if kubernetes_namespaces
+            else [],
+            kubernetes_ingress_external_address=kubernetes_ingress_external_address,
+            kubernetes_zones=kubernetes_zones.split(",") if kubernetes_zones else [],
+            kubernetes_dataplane_identity=kubernetes_dataplane_identity,
             private_network=private_network,
             cluster_management_stack_version=ClusterManagementStackVersions.V2,
             yes=yes,
             skip_verifications=skip_verifications,
             auto_add_user=enable_auto_add_user,
+            external_id=external_id,
         )
     elif provider == "gcp":
         # Keep the parameter naming ({resource}_name or {resource}_id) consistent with GCP to reduce confusion for customers
         # Check if all required parameters are provided
         # memorystore_instance_name and host_project_id are optional for GCP clouds
         for resource in [
-            (project_id, "--project-id"),
-            (vpc_name, "--vpc-name"),
-            (subnet_names, "--subnet-names"),
-            (filestore_instance_id, "--filestore-instance-id"),
-            (filestore_location, "--filestore-location"),
-            (anyscale_service_account_email, "--anyscale-service-account-email"),
-            (instance_service_account_email, "--instance-service-account-email"),
-            (provider_name, "--provider-name"),
-            (firewall_policy_names, "--firewall-policy-names"),
-            (cloud_storage_bucket_name, "--cloud-storage-bucket-name"),
+            (project_id, "--project-id", (ComputeStack.VM, ComputeStack.K8S)),
+            (vpc_name, "--vpc-name", (ComputeStack.VM)),
+            (subnet_names, "--subnet-names", (ComputeStack.VM)),
+            (filestore_instance_id, "--filestore-instance-id", (ComputeStack.VM)),
+            (filestore_location, "--filestore-location", (ComputeStack.VM)),
+            (
+                anyscale_service_account_email,
+                "--anyscale-service-account-email",
+                (ComputeStack.VM, ComputeStack.K8S),
+            ),
+            (
+                instance_service_account_email,
+                "--instance-service-account-email",
+                (ComputeStack.VM),
+            ),
+            (provider_name, "--provider-name", (ComputeStack.VM, ComputeStack.K8S)),
+            (firewall_policy_names, "--firewall-policy-names", (ComputeStack.VM)),
+            (
+                cloud_storage_bucket_name,
+                "--cloud-storage-bucket-name",
+                (ComputeStack.VM, ComputeStack.K8S),
+            ),
+            (kubernetes_namespaces, "--kubernetes-namespaces", (ComputeStack.K8S)),
+            (
+                kubernetes_ingress_external_address,
+                "--kubernetes-ingress-external-address",
+                (ComputeStack.K8S),
+            ),
+            (kubernetes_zones, "--kubernetes-zones", (ComputeStack.K8S)),
+            (
+                kubernetes_dataplane_identity,
+                "--kubernetes-dataplane-identity",
+                (ComputeStack.K8S),
+            ),
         ]:
-            if resource[0] is None:
+            if compute_stack in resource[2] and resource[0] is None:
                 missing_args.append(resource[1])
 
         if len(missing_args) > 0:
@@ -593,19 +792,28 @@ def register_cloud(  # noqa: PLR0913, PLR0912
         CloudController().register_gcp_cloud(
             region=region,
             name=name,
+            compute_stack=compute_stack,
             project_id=project_id,
             vpc_name=vpc_name,
-            subnet_names=subnet_names.split(","),
+            subnet_names=subnet_names.split(",") if subnet_names else [],
             filestore_instance_id=filestore_instance_id,
             filestore_location=filestore_location,
             anyscale_service_account_email=anyscale_service_account_email,
             instance_service_account_email=instance_service_account_email,
             # TODO (allenyin): use provider_name instead of provider_id everywhere.
             provider_id=provider_name,
-            firewall_policy_names=firewall_policy_names.split(","),
+            firewall_policy_names=firewall_policy_names.split(",")
+            if firewall_policy_names
+            else [],
             cloud_storage_bucket_name=cloud_storage_bucket_name,
             memorystore_instance_name=memorystore_instance_name,
             host_project_id=host_project_id,
+            kubernetes_namespaces=kubernetes_namespaces.split(",")
+            if kubernetes_namespaces
+            else [],
+            kubernetes_ingress_external_address=kubernetes_ingress_external_address,
+            kubernetes_zones=kubernetes_zones.split(",") if kubernetes_zones else [],
+            kubernetes_dataplane_identity=kubernetes_dataplane_identity,
             functional_verify=functional_verify,
             private_network=private_network,
             cluster_management_stack_version=ClusterManagementStackVersions.V2,
@@ -613,6 +821,34 @@ def register_cloud(  # noqa: PLR0913, PLR0912
             skip_verifications=skip_verifications,
             auto_add_user=enable_auto_add_user,
         )
+    elif provider == "generic":
+        # For the 'generic' provider type, for the time being, most fields are optional; only 'name', 'provider', 'compute-stack', and 'kubernetes-namespaces' are required.
+        if not name:
+            raise click.ClickException("Please provide a value for --name.")
+
+        if compute_stack != ComputeStack.K8S:
+            raise click.ClickException(
+                "--compute-stack=k8s must be passed for registering generic Anyscale clouds."
+            )
+
+        if not kubernetes_namespaces:
+            raise click.ClickException(
+                "Please provide at least one namespace to `--kubernetes-namespaces`."
+            )
+
+        CloudController().register_generic_cloud(
+            name=name,
+            auto_add_user=enable_auto_add_user,
+            region=region,
+            cloud_storage_bucket_name=cloud_storage_bucket_name,
+            cloud_storage_bucket_endpoint=cloud_storage_bucket_endpoint,
+            nfs_mount_targets=list(nfs_mount_target) if nfs_mount_target else [],
+            nfs_mount_path=nfs_mount_path,
+            kubernetes_namespaces=kubernetes_namespaces.split(","),
+            kubernetes_ingress_external_address=kubernetes_ingress_external_address,
+            kubernetes_zones=kubernetes_zones.split(",") if kubernetes_zones else [],
+        )
+
     else:
         raise click.ClickException(
             f"Invalid Cloud provider: {provider}. Available providers are [aws, gcp]."
@@ -662,7 +898,10 @@ def cloud_verify(
     )
 
 
-@cloud_cli.command(name="edit", help="Edit anyscale cloud resource.")
+@cloud_cli.command(
+    name="edit",
+    help="Edit registered cloud resource on Anyscale. Only applicable for anyscale registered clouds.",
+)
 @click.argument("cloud-name", required=False)
 @click.option("--name", "-n", help="Edit cloud by name.", type=str)
 @click.option(
@@ -794,5 +1033,5 @@ def cloud_edit(  # noqa: PLR0913
         )
     else:
         raise click.ClickException(
-            "Please provide at least one of the following arguments: --aws-s3-id, --aws-efs-id, --aws-efs-mount-target-ip, --memorydb-cluster-id, --gcp-filestore-instance-id, --gcp-filestore-location, --gcp-cloud-storage-bucket-name, --memorystore-instance-name."
+            "Please provide at least one of the following arguments: --aws-s3-id, --aws-efs-id, --aws-efs-mount-target-ip, --memorydb-cluster-id, --gcp-filestore-instance-id, --gcp-filestore-location, --gcp-cloud-storage-bucket-name, --memorystore-instance-name, --enable-auto-add-user, --disable-auto-add-user."
         )
