@@ -7,11 +7,12 @@ import click
 from typing_extensions import Literal
 import yaml
 
+from anyscale._private.models.image_uri import ImageURI
 from anyscale.cli_logger import BlockLogger
-from anyscale.commands.util import convert_kv_strings_to_dict
+from anyscale.commands.util import convert_kv_strings_to_dict, LegacyAnyscaleCommand
 from anyscale.controllers.service_controller import ServiceController
 import anyscale.service
-from anyscale.service import ServiceConfig, ServiceStatus
+from anyscale.service import ServiceConfig, ServiceState, ServiceStatus
 from anyscale.util import validate_non_negative_arg
 
 
@@ -42,7 +43,7 @@ def _read_name_from_config_file(path: str):
 
 
 @service_cli.command(
-    name="deploy", short_help="Deploy or update a service.", hidden=True,
+    name="deploy", help="Deploy or update a service.",
 )
 @click.argument("import_path", type=str, required=False, default=None)
 @click.argument("arguments", nargs=-1, required=False)
@@ -69,6 +70,14 @@ def _read_name_from_config_file(path: str):
     type=str,
     help="Container image to use for the service. This cannot be changed when using --in-place and is exclusive with --containerfile. When running in a workspace, this defaults to the image of the workspace.",
 )
+@click.option(
+    "--registry-login-secret",
+    required=False,
+    default=None,
+    type=str,
+    help="Name or identifier of the secret containing credentials to authenticate to the docker registry hosting the image. "
+    "This can only be used when 'image_uri' is specified and the image is not hosted on Anyscale.",
+)
 @click.option(
     "--containerfile",
     required=False,
@@ -76,6 +85,13 @@ def _read_name_from_config_file(path: str):
     type=str,
     help="Path to a containerfile to build the image to use for the service. This cannot be changed when using --in-place and is exclusive with --image-uri.",
 )
+@click.option(
+    "--ray-version",
+    required=False,
+    default=None,
+    type=str,
+    help="The Ray version (X.Y.Z) to the image specified by --image-uri. This is only used when --image-uri is provided. If not provided, the latest Ray version will be used.",
+)
 @click.option(
     "--compute-config",
     required=False,
@@ -144,12 +160,28 @@ def _read_name_from_config_file(path: str):
     type=str,
     help="Python modules to be available for import in the Ray workers. Each entry must be a path to a local directory.",
 )
+@click.option(
+    "--cloud",
+    required=False,
+    default=None,
+    type=str,
+    help="The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
+)
+@click.option(
+    "--project",
+    required=False,
+    default=None,
+    type=str,
+    help="Named project to use for the service. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
+)
 def deploy(  # noqa: PLR0912, PLR0913 C901
     config_file: Optional[str],
     import_path: Optional[str],
     arguments: Tuple[str],
     name: Optional[str],
     image_uri: Optional[str],
+    registry_login_secret: Optional[str],
+    ray_version: Optional[str],
     containerfile: Optional[str],
     compute_config: Optional[str],
     working_dir: Optional[str],
@@ -160,6 +192,8 @@ def deploy(  # noqa: PLR0912, PLR0913 C901
     max_surge_percent: Optional[int],
     env: Optional[Tuple[str]],
     py_module: Tuple[str],
+    cloud: Optional[str],
+    project: Optional[str],
 ):
     """Deploy or update a service.
 
@@ -212,12 +246,30 @@ def deploy(  # noqa: PLR0912, PLR0913 C901
             "Only one of '--containerfile' and '--image-uri' can be provided."
         )
 
+    if ray_version and (not image_uri and not containerfile):
+        raise click.ClickException(
+            "Ray version can only be used with an image or containerfile.",
+        )
+
+    if registry_login_secret and (
+        not image_uri or ImageURI.from_str(image_uri).is_cluster_env_image()
+    ):
+        raise click.ClickException(
+            "Registry login secret can only be used with an image that is not hosted on Anyscale."
+        )
+
     if name is not None:
         config = config.options(name=name)
 
     if image_uri is not None:
         config = config.options(image_uri=image_uri)
 
+    if registry_login_secret is not None:
+        config = config.options(registry_login_secret=registry_login_secret)
+
+    if ray_version is not None:
+        config = config.options(ray_version=ray_version)
+
     if containerfile is not None:
         config = config.options(containerfile=containerfile)
 
@@ -246,6 +298,11 @@ def deploy(  # noqa: PLR0912, PLR0913 C901
                 )
         config = config.options(py_modules=[*py_module])
 
+    if cloud is not None:
+        config = config.options(cloud=cloud)
+    if project is not None:
+        config = config.options(project=project)
+
     anyscale.service.deploy(
         config,
         in_place=in_place,
@@ -255,15 +312,10 @@ def deploy(  # noqa: PLR0912, PLR0913 C901
 
 
 @service_cli.command(
-    name="status", help="Get the status of a service.", hidden=True,
+    name="status", help="Get the status of a service.",
 )
 @click.option(
-    "-n",
-    "--name",
-    required=False,
-    default=None,
-    type=str,
-    help="Name of the service. When running in a workspace, this defaults to the workspace name.",
+    "-n", "--name", required=False, default=None, type=str, help="Name of the service.",
 )
 @click.option(
     "-f",
@@ -273,6 +325,20 @@ def deploy(  # noqa: PLR0912, PLR0913 C901
     type=str,
     help="Path to a YAML config file to read the name from.",
 )
+@click.option(
+    "--cloud",
+    required=False,
+    default=None,
+    type=str,
+    help="The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
+)
+@click.option(
+    "--project",
+    required=False,
+    default=None,
+    type=str,
+    help="Named project to use for the service. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
+)
 @click.option(
     "-j",
     "--json",
@@ -287,7 +353,14 @@ def deploy(  # noqa: PLR0912, PLR0913 C901
     default=False,
     help="Include verbose details in the status.",
 )
-def status(name: Optional[str], config_file: Optional[str], json: bool, verbose: bool):
+def status(
+    name: Optional[str],
+    config_file: Optional[str],
+    cloud: Optional[str],
+    project: Optional[str],
+    json: bool,
+    verbose: bool,
+):
     if name is not None and config_file is not None:
         raise click.ClickException(
             "Only one of '--name' and '--config-file' can be provided."
@@ -296,7 +369,14 @@ def status(name: Optional[str], config_file: Optional[str], json: bool, verbose:
     if config_file is not None:
         name = _read_name_from_config_file(config_file)
 
-    status: ServiceStatus = anyscale.service.status(name=name)
+    if name is None:
+        raise click.ClickException(
+            "Service name must be provided using '--name' or in a config file using '-f'."
+        )
+
+    status: ServiceStatus = anyscale.service.status(
+        name=name, cloud=cloud, project=project
+    )
     status_dict = status.to_dict()
     if not verbose:
         # TODO(edoakes): consider adding this as an API on the model itself if it
@@ -313,7 +393,83 @@ def status(name: Optional[str], config_file: Optional[str], json: bool, verbose:
 
 
 @service_cli.command(
-    name="rollout", help="Roll out a service.",
+    name="wait", help="Wait for a service to enter a target state.",
+)
+@click.option(
+    "-n", "--name", required=False, default=None, type=str, help="Name of the service.",
+)
+@click.option(
+    "-f",
+    "--config-file",
+    required=False,
+    default=None,
+    type=str,
+    help="Path to a YAML config file to read the name from.",
+)
+@click.option(
+    "--cloud",
+    required=False,
+    default=None,
+    type=str,
+    help="The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
+)
+@click.option(
+    "--project",
+    required=False,
+    default=None,
+    type=str,
+    help="Named project to use for the service. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
+)
+@click.option(
+    "-s",
+    "--state",
+    default=str(ServiceState.RUNNING),
+    help="The ServiceState to wait for the service to reach. Defaults to RUNNING.",
+)
+@click.option(
+    "-t",
+    "--timeout-s",
+    default=600,
+    type=float,
+    help="Timeout to wait for the service to reach the target state. Defaults to 600s (10min).",
+)
+def wait(
+    name: Optional[str],
+    config_file: Optional[str],
+    cloud: Optional[str],
+    project: Optional[str],
+    state: str,
+    timeout_s: int,
+):
+    try:
+        state = ServiceState.validate(state)
+    except ValueError as e:
+        raise click.ClickException(str(e))
+
+    if name is not None and config_file is not None:
+        raise click.ClickException(
+            "Only one of '--name' and '--config-file' can be provided."
+        )
+
+    if config_file is not None:
+        name = _read_name_from_config_file(config_file)
+
+    if name is None:
+        raise click.ClickException(
+            "Service name must be provided using '--name' or in a config file using '-f'."
+        )
+
+    anyscale.service.wait(
+        name=name, cloud=cloud, project=project, state=state, timeout_s=timeout_s
+    )
+
+
+@service_cli.command(
+    name="rollout",
+    help="Roll out a service.",
+    cls=LegacyAnyscaleCommand,
+    new_prefix="anyscale service",
+    new_cli=deploy,
 )
 @click.option(
     "-f",
@@ -392,6 +548,7 @@ def rollout(  # noqa: PLR0913
     )
 
 
+# TODO(mowen): Add cloud support for this when we refactor to new SDK method
 @service_cli.command(name="list", help="Display information about services.")
 @click.option(
     "--name", "-n", required=False, default=None, help="Filter by service name."
@@ -437,6 +594,7 @@ def list(  # noqa: A001
     )
 
 
+# TODO(mowen): Add cloud support for this when we refactor to new SDK method
 @service_cli.command(name="rollback", help="Roll back a service.")
 @click.option(
     "--service-id", "--id", default=None, help="ID of service.",
@@ -463,10 +621,7 @@ def rollback(
     config_file: Optional[str],
     max_surge_percent: Optional[int],
 ):
-    """Perform a rollback for a service that is currently in a rollout.
-
-    This *only* applies to v2 services.
-    """
+    """Perform a rollback for a service that is currently in a rollout."""
     service_controller = ServiceController()
     service_id = service_controller.get_service_id(
         service_id=service_id,
@@ -477,6 +632,7 @@ def rollback(
     service_controller.rollback(service_id, max_surge_percent)
 
 
+# TODO(mowen): Add cloud support for this when we refactor to new SDK method
 @service_cli.command(name="terminate", help="Terminate a service.")
 @click.option(
     "--service-id", "--id", required=False, help="ID of service.",
