@@ -3,6 +3,58 @@ from typing import Dict, Tuple
 import click
 
 
+class AnyscaleCommand(click.Command):
+    """
+    AnyscaleCommand is a subclass of click.Command that allows for the addition of
+    CLI examples.
+
+    :param __doc_cli_example__: A string that represents CLI examples.
+    """
+
+    def __init__(self, *args, **kwargs):
+        self.__doc_cli_example__ = kwargs.pop("example", None)
+        self.__is_alpha__ = kwargs.pop("is_alpha", False)
+        self.__is_beta__ = kwargs.pop("is_beta", False)
+        super().__init__(*args, **kwargs)
+
+    @property
+    def is_alpha(self) -> bool:
+        return self.__is_alpha__
+
+    @property
+    def is_beta(self) -> bool:
+        return self.__is_beta__
+
+
+class LegacyAnyscaleCommand(click.Command):
+    """
+    LegacyAnyscaleCommand is a subclass of click.Command that allows for the addition of
+    upgrade information.
+
+    :param __doc_new_cli__: The CLI command that the legacy command should be upgraded to.
+    :param __doc_new_prefix__ : The prefix of the new CLI command.
+    """
+
+    def __init__(self, *args, **kwargs):
+        self.__doc_new_cli__ = kwargs.pop("new_cli", None)
+        self.__doc_new_prefix__ = kwargs.pop("new_prefix", None)
+        self.__doc_legacy_prefix__ = kwargs.pop("legacy_prefix", None)
+        self.__is_limited_support__ = kwargs.pop("is_limited_support", False)
+        super().__init__(*args, **kwargs)
+
+    def get_new_cli(self) -> AnyscaleCommand:
+        return self.__doc_new_cli__
+
+    def get_new_prefix(self) -> str:
+        return self.__doc_new_prefix__
+
+    def get_legacy_prefix(self) -> str:
+        return self.__doc_legacy_prefix__
+
+    def is_limited_support(self) -> bool:
+        return self.__is_limited_support__
+
+
 # Take from https://stackoverflow.com/questions/51846634/click-dynamic-defaults-for-prompts-based-on-other-options
 class OptionPromptNull(click.Option):
     """
