@@ -21,19 +21,12 @@ from anyscale.controllers.base_controller import BaseController
 from anyscale.sdk.anyscale_client.models.cluster_compute_config import (
     ClusterComputeConfig,
 )
-from anyscale.sdk.anyscale_client.models.cluster_computes_query import (
-    ClusterComputesQuery,
-)
-from anyscale.sdk.anyscale_client.models.clustercompute_list_response import (
-    ClustercomputeListResponse,
-)
 from anyscale.sdk.anyscale_client.models.create_cluster_compute import (
     CreateClusterCompute,
 )
 from anyscale.sdk.anyscale_client.models.create_cluster_compute_config import (
     CreateClusterComputeConfig,
 )
-from anyscale.sdk.anyscale_client.models.text_query import TextQuery
 from anyscale.util import get_endpoint
 from anyscale.utils.entity_arg_utils import EntityType, IdBasedEntity, NameBasedEntity
 from anyscale.utils.name_utils import gen_valid_name
@@ -240,38 +233,6 @@ class ComputeConfigController(BaseController):
         if enable_compute_config_versioning:
             log.info(f"Compute config version: {cluster_compute_version}.")
 
-    def delete(
-        self, cluster_compute_name: Optional[str], id: Optional[str]  # noqa: A002
-    ) -> None:
-        """
-        TODO: shawnp. Remove once archive functionality is enabled on prod.
-        Deletes the cluster compute with the given name or id.
-        Exactly one of cluster_compute_name or id must be provided.
-        """
-
-        if int(bool(cluster_compute_name)) + int(bool(id)) != 1:
-            raise ClickException(
-                "Not deleted. Please provide exactly one of: compute config name, id."
-            )
-
-        if id:
-            self.anyscale_api_client.get_cluster_compute(id)
-            cluster_compute_id = id
-        else:
-            # find the cluster compute id from the name
-            query = ClusterComputesQuery(name=TextQuery(equals=cluster_compute_name))
-            query_response: ClustercomputeListResponse = self.anyscale_api_client.search_cluster_computes(
-                query
-            )
-            cluster_results = query_response.results
-            if len(cluster_results) != 1:
-                raise ClickException(
-                    f"Not deleted. No compute config exists with the name {cluster_compute_name}."
-                )
-            cluster_compute_id = cluster_results[0].id
-        self.anyscale_api_client.delete_cluster_compute(cluster_compute_id)
-        log.info("Compute config deleted.")
-
     def archive(
         self, compute_config_entity: Union[IdBasedEntity, NameBasedEntity]
     ) -> None:
