@@ -26,6 +26,8 @@ def format_clouds_output_json(clouds: List[Cloud]) -> str:
                 "name": cloud.name,
                 "provider": cloud.provider,
                 "region": cloud.region,
+                "added_date": cloud.created_at.strftime("%m/%d/%Y"),
+                "default": cloud.is_default,
                 "credentials": cloud.credentials,
             }
         )
@@ -36,11 +38,27 @@ def format_clouds_output_table(clouds: List[Cloud]) -> str:
     table_rows = []
     for cloud in clouds:
         table_rows.append(
-            [cloud.id, cloud.name, cloud.provider, cloud.region, cloud.credentials]
+            [
+                cloud.id,
+                cloud.name,
+                cloud.provider,
+                cloud.region,
+                cloud.created_at.strftime("%m/%d/%Y"),
+                cloud.is_default,
+                cloud.credentials,
+            ]
         )
     table = tabulate.tabulate(
         table_rows,
-        headers=["ID", "name", "PROVIDER", "REGION", "CREDENTIALS"],
+        headers=[
+            "ID",
+            "NAME",
+            "PROVIDER",
+            "REGION",
+            "ADDED DATE",
+            "DEFAULT",
+            "CREDENTIALS",
+        ],
         tablefmt="plain",
     )
 
