@@ -239,6 +239,32 @@ def get_proj_id_from_name(
     return str(selected_project.id)
 
 
+def get_project_id_for_cloud_from_name(
+    project_name: str,
+    parent_cloud_id: str,
+    api_client: Optional[DefaultApi] = None,
+    anyscale_api_client: Optional[DefaultApi] = None,
+) -> str:
+    if api_client is None:
+        api_client = get_auth_api_client().api_client
+
+    if project_name == "default":  # get default project
+        return get_default_project(api_client, anyscale_api_client, parent_cloud_id).id
+
+    existing_projects = api_client.find_project_by_project_name_api_v2_projects_find_by_name_get(
+        project_name
+    ).results
+    if len(existing_projects) == 0:
+        raise ClickException(f"Project '{project_name}' was not found.")
+    else:
+        for project in existing_projects:
+            if project.parent_cloud_id == parent_cloud_id:
+                return project.id
+        raise ClickException(
+            f"Project '{project_name}' under cloud '{parent_cloud_id}' was not found."
+        )
+
+
 def get_parent_cloud_id_and_name_of_project(
     project_id: str, api_client: Optional[DefaultApi] = None,
 ) -> Optional[Tuple[str, str]]:
