@@ -1,17 +1,19 @@
 import copy
-from typing import Any, Dict, Optional, Union
+from typing import Any, Dict, Optional
 
-from anyscale._private.models.image_uri import ImageURI
 from anyscale._private.workload import WorkloadSDK
-from anyscale.cli_logger import BlockLogger
+from anyscale.compute_config.models import ComputeConfig
 from anyscale.sdk.anyscale_client.models import (
     AccessConfig,
     ApplyServiceModel,
+    GrpcProtocolConfig,
     ProductionServiceV2VersionModel,
+    Protocols,
     RayGCSExternalStorageConfig as APIRayGCSExternalStorageConfig,
     ServiceConfig as ExternalAPIServiceConfig,
     ServiceEventCurrentState,
     ServiceModel,
+    TracingConfig as APITracingConfg,
 )
 from anyscale.service.models import (
     RayGCSExternalStorageConfig,
@@ -19,6 +21,7 @@ from anyscale.service.models import (
     ServiceState,
     ServiceStatus,
     ServiceVersionStatus,
+    TracingConfig,
 )
 from anyscale.utils.runtime_env import parse_requirements_file
 from anyscale.utils.workspace_notification import (
@@ -27,10 +30,7 @@ from anyscale.utils.workspace_notification import (
 )
 
 
-logger = BlockLogger()
-
-
-class ServiceSDK(WorkloadSDK):
+class PrivateServiceSDK(WorkloadSDK):
     SERVICE_VERSION_ID_TRUNCATED_LEN = 8
 
     def _override_application_runtime_envs(
@@ -125,12 +125,12 @@ class ServiceSDK(WorkloadSDK):
             f"View the service in the UI: '{self.client.get_service_ui_url(service.id)}'"
         )
         self.logger.info(
-            "Query the service once it's running using the following curl command:"
+            "Query the service once it's running using the following curl command (add the path you want to query):"
         )
         auth_token_header = (
             ""
             if service.auth_token is None
-            else f"-H 'Authorization: Bearer {service.auth_token}' "
+            else f'-H "Authorization: Bearer {service.auth_token}" '
         )
         self.logger.info(f"curl {auth_token_header}{service.base_url}/")
 
@@ -145,6 +145,19 @@ class ServiceSDK(WorkloadSDK):
 
         return ray_serve_config
 
+    def _build_grpc_protocol_config(self, config: ServiceConfig) -> GrpcProtocolConfig:
+        enabled = (
+            config.grpc_options is not None
+            and len(config.grpc_options.get("service_names", [])) > 0
+        )
+        service_names = (
+            config.grpc_options.get("service_names", [])
+            if config.grpc_options is not None
+            else []
+        )
+        # Since config doesn't allow passing a port, keeping it as default port.
+        return GrpcProtocolConfig(enabled=enabled, service_names=service_names)
+
     def _build_apply_service_model_for_in_place_update(  # noqa: PLR0912
         self,
         name: str,
@@ -195,9 +208,11 @@ class ServiceSDK(WorkloadSDK):
         if any(
             [
                 config.image_uri is not None,
+                config.registry_login_secret is not None,
                 config.containerfile is not None,
                 config.compute_config is not None,
                 config.ray_gcs_external_storage_config is not None,
+                config.tracing_config is not None,
             ]
         ):
             self.logger.warning(
@@ -215,17 +230,23 @@ class ServiceSDK(WorkloadSDK):
             cloud_id=cloud_id,
             workspace_requirements_path=self.client.get_workspace_requirements_path(),
         )
+
+        project_id = self.client.get_project_id(
+            parent_cloud_id=cloud_id, name=config.project
+        )
         return ApplyServiceModel(
             name=name,
-            project_id=self.client.get_project_id(parent_cloud_id=cloud_id),
+            project_id=project_id,
             ray_serve_config=self._build_ray_serve_config(config),
             build_id=existing_config.build_id,
             compute_config_id=existing_config.compute_config_id,
             rollout_strategy="IN_PLACE",
             config=ExternalAPIServiceConfig(
                 access=AccessConfig(use_bearer_token=query_auth_token_enabled),
+                protocols=Protocols(grpc=self._build_grpc_protocol_config(config)),
             ),
             ray_gcs_external_storage_config=existing_config.ray_gcs_external_storage_config,
+            tracing_config=existing_config.tracing_config,
         )
 
     def _build_apply_service_model_for_rollout(  # noqa: PLR0912
@@ -241,23 +262,27 @@ class ServiceSDK(WorkloadSDK):
 
         build_id = None
         if config.containerfile is not None:
-            build_id = self.client.get_cluster_env_build_id_from_containerfile(
-                cluster_env_name=f"image-for-service-{name}",
+            build_id = self._image_sdk.build_image_from_containerfile(
+                name=f"image-for-service-{name}",
                 containerfile=self.get_containerfile_contents(config.containerfile),
+                anonymous=True,
+                ray_version=config.ray_version,
             )
         elif config.image_uri is not None:
-            build_id = self.client.get_cluster_env_build_id_from_image_uri(
-                image_uri=ImageURI.from_str(config.image_uri)
+            build_id = self._image_sdk.registery_image(
+                image_uri=config.image_uri,
+                registry_login_secret=config.registry_login_secret,
+                ray_version=config.ray_version,
             )
 
-        if self.enable_image_build_for_tracked_requirements:
+        if self._image_sdk.enable_image_build_for_tracked_requirements:
             requirements_path_to_be_populated_in_runtime_env = None
             requirements_path = self.client.get_workspace_requirements_path()
             if requirements_path is not None:
                 requirements = parse_requirements_file(requirements_path)
                 if requirements:
-                    build_id = self.client.build_image_from_requirements(
-                        cluster_env_name=f"image-for-service-{name}",
+                    build_id = self._image_sdk.build_image_from_requirements(
+                        name=f"image-for-service-{name}",
                         base_build_id=self.client.get_default_build_id(),
                         requirements=requirements,
                     )
@@ -269,7 +294,13 @@ class ServiceSDK(WorkloadSDK):
         if build_id is None:
             build_id = self.client.get_default_build_id()
 
-        compute_config_id, cloud_id = self.resolve_compute_config(config.compute_config)
+        compute_config_id, cloud_id = self.resolve_compute_config_and_cloud_id(
+            compute_config=config.compute_config, cloud=config.cloud
+        )
+
+        project_id = self.client.get_project_id(
+            parent_cloud_id=cloud_id, name=config.project
+        )
         if (
             existing_service is not None
             and existing_service.primary_version is not None
@@ -319,9 +350,21 @@ class ServiceSDK(WorkloadSDK):
                 ray_gcs_external_storage_config.redis_certificate_path = (
                     config.ray_gcs_external_storage_config.certificate_path
                 )
+
+        tracing_config = None
+        if config.tracing_config is not None:
+            assert isinstance(config.tracing_config, TracingConfig)
+            tracing_config = APITracingConfg(enabled=config.tracing_config.enabled,)
+            if config.tracing_config.exporter_import_path is not None:
+                tracing_config.exporter_import_path = (
+                    config.tracing_config.exporter_import_path
+                )
+            if config.tracing_config.sampling_ratio is not None:
+                tracing_config.sampling_ratio = config.tracing_config.sampling_ratio
+
         return ApplyServiceModel(
             name=name,
-            project_id=self.client.get_project_id(parent_cloud_id=cloud_id),
+            project_id=project_id,
             ray_serve_config=self._build_ray_serve_config(config),
             build_id=build_id,
             compute_config_id=compute_config_id,
@@ -330,8 +373,10 @@ class ServiceSDK(WorkloadSDK):
             rollout_strategy="ROLLOUT",
             config=ExternalAPIServiceConfig(
                 access=AccessConfig(use_bearer_token=config.query_auth_token_enabled),
+                protocols=Protocols(grpc=self._build_grpc_protocol_config(config)),
             ),
             ray_gcs_external_storage_config=ray_gcs_external_storage_config,
+            tracing_config=tracing_config,
         )
 
     def deploy(
@@ -341,7 +386,7 @@ class ServiceSDK(WorkloadSDK):
         in_place: bool = False,
         canary_percent: Optional[int] = None,
         max_surge_percent: Optional[int] = None,
-    ):
+    ) -> str:
         if not isinstance(in_place, bool):
             raise TypeError("in_place must be a bool.")
 
@@ -358,7 +403,9 @@ class ServiceSDK(WorkloadSDK):
                 raise ValueError("max_surge_percent must be between 0 and 100.")
 
         name = config.name or self._get_default_name()
-        existing_service: Optional[ServiceModel] = self.client.get_service(name)
+        existing_service: Optional[ServiceModel] = self.client.get_service(
+            name=name, cloud=config.cloud, project=config.project
+        )
         if existing_service is None:
             self.logger.info(f"Starting new service '{name}'.")
         elif existing_service.current_state == ServiceEventCurrentState.TERMINATED:
@@ -386,38 +433,67 @@ class ServiceSDK(WorkloadSDK):
         service: ServiceModel = self.client.rollout_service(model)
         self._log_deployed_service_info(service, canary_percent=canary_percent)
 
-    def rollback(
-        self, name: Optional[str] = None, *, max_surge_percent: Optional[int] = None,
-    ):
+        return service.id
+
+    def _resolve_to_service_model(
+        self,
+        *,
+        name: Optional[str] = None,
+        cloud: Optional[str] = None,
+        project: Optional[str] = None,
+    ) -> ServiceModel:
         if name is None:
             name = self._get_default_name()
 
-        model: Optional[ServiceModel] = self.client.get_service(name)
+        model: Optional[ServiceModel] = self.client.get_service(
+            name=name, cloud=cloud, project=project
+        )
         if model is None:
             raise RuntimeError(f"Service with name '{name}' was not found.")
 
+        return model
+
+    def rollback(
+        self,
+        name: Optional[str] = None,
+        *,
+        cloud: Optional[str] = None,
+        project: Optional[str] = None,
+        max_surge_percent: Optional[int] = None,
+    ) -> str:
+        model = self._resolve_to_service_model(name=name, cloud=cloud, project=project)
+
         self.client.rollback_service(model.id, max_surge_percent=max_surge_percent)
 
-    def terminate(self, name: Optional[str] = None):
-        if name is None:
-            name = self._get_default_name()
+        return model.id
 
-        model: Optional[ServiceModel] = self.client.get_service(name)
-        if model is None:
-            raise RuntimeError(f"Service with name '{name}' was not found.")
+    def terminate(
+        self,
+        name: Optional[str] = None,
+        *,
+        cloud: Optional[str] = None,
+        project: Optional[str] = None,
+    ) -> str:
+        model = self._resolve_to_service_model(name=name, cloud=cloud, project=project)
 
         self.client.terminate_service(model.id)
 
+        return model.id
+
     def _service_version_model_to_status(
         self,
         model: ProductionServiceV2VersionModel,
         *,
         service_name: str,
+        project_id: str,
         query_auth_token_enabled: bool,
     ) -> ServiceVersionStatus:
-        image_uri = self.client.get_cluster_env_build_image_uri(model.build_id)
+        image_uri = self._image_sdk.get_image_uri_from_build_id(model.build_id)
         if image_uri is None:
             raise RuntimeError(f"Failed to get image URI for ID {model.build_id}.")
+        image_build = self._image_sdk.get_image_build(model.build_id)
+        if image_build is None:
+            raise RuntimeError(f"Failed to get image build for ID {model.build_id}.")
 
         ray_gcs_external_storage_config = None
         if model.ray_gcs_external_storage_config is not None:
@@ -427,6 +503,16 @@ class ServiceSDK(WorkloadSDK):
                 certificate_path=model.ray_gcs_external_storage_config.redis_certificate_path,
             )
 
+        project = self.client.get_project(project_id)
+        compute_config = self.get_user_facing_compute_config(model.compute_config_id)
+        tracing_config = None
+        if model.tracing_config is not None:
+            tracing_config = TracingConfig(
+                enabled=model.tracing_config.enabled,
+                exporter_import_path=model.tracing_config.exporter_import_path,
+                sampling_ratio=model.tracing_config.sampling_ratio,
+            )
+
         return ServiceVersionStatus(
             # NOTE(edoakes): the "version ID" exposed in the UI and tagged in the metrics is
             # not actually the DB ID, but a truncated version of it. We should store this
@@ -439,17 +525,40 @@ class ServiceSDK(WorkloadSDK):
                 name=service_name,
                 applications=model.ray_serve_config["applications"],
                 image_uri=str(image_uri),
-                compute_config=self.get_user_facing_compute_config(
-                    model.compute_config_id
-                ),
+                compute_config=compute_config,
+                registry_login_secret=image_build.registry_login_secret,
                 query_auth_token_enabled=query_auth_token_enabled,
                 http_options=model.ray_serve_config.get("http_options", None),
                 grpc_options=model.ray_serve_config.get("grpc_options", None),
                 logging_config=model.ray_serve_config.get("logging_config", None),
                 ray_gcs_external_storage_config=ray_gcs_external_storage_config,
+                cloud=compute_config.cloud
+                if compute_config and isinstance(compute_config, ComputeConfig)
+                else None,
+                project=project.name
+                if project is not None and project.name != "default"
+                else None,
+                tracing_config=tracing_config,
             ),
         )
 
+    def _model_state_to_state(
+        self, model_state: ServiceEventCurrentState,
+    ) -> ServiceState:
+        # If we add a new state to the backend, old clients may not recognize it.
+        # Rather than erroring out and causing old code to crash, return UNKNOWN.
+        state: ServiceState = ServiceState(ServiceState.UNKNOWN)
+        try:
+            state = ServiceState(model_state)
+        except ValueError:
+            self.logger.warning(
+                f"Got unrecognized state: '{model_state}'. "
+                "You likely need to update the 'anyscale' package. "
+                "If you still see this message after upgrading, contact Anyscale support."
+            )
+
+        return state
+
     def _service_model_to_status(self, model: ServiceModel) -> ServiceStatus:
         # TODO(edoakes): for some reason the primary_version is populated
         # when the service is terminated. This should be fixed in the backend.
@@ -465,6 +574,7 @@ class ServiceSDK(WorkloadSDK):
             primary_version = self._service_version_model_to_status(
                 model.primary_version,
                 service_name=model.name,
+                project_id=model.project_id,
                 query_auth_token_enabled=query_auth_token_enabled,
             )
 
@@ -473,37 +583,70 @@ class ServiceSDK(WorkloadSDK):
             canary_version = self._service_version_model_to_status(
                 model.canary_version,
                 service_name=model.name,
+                project_id=model.project_id,
                 query_auth_token_enabled=query_auth_token_enabled,
             )
 
-        # If we add a new state to the backend, old clients may not recognize it.
-        # Rather than erroring out and causing old code to crash, return UNKNOWN.
-        state: Union[str, ServiceState] = ServiceState.UNKNOWN
-        try:
-            state = ServiceState(model.current_state)
-        except ValueError:
-            self.logger.warning(
-                f"Got unrecognized state: '{model.current_state}'. "
-                "You likely need to update the 'anyscale' package. "
-                "If you still see this message after upgrading, contact Anyscale support."
-            )
-
         return ServiceStatus(
             id=model.id,
             name=model.name,
-            state=state,
+            state=self._model_state_to_state(model.current_state),
             query_url=model.base_url,
             query_auth_token=model.auth_token,
             primary_version=primary_version,
             canary_version=canary_version,
         )
 
-    def status(self, name: Optional[str] = None) -> ServiceStatus:
-        if name is None:
-            name = self._get_default_name()
-
-        model: Optional[ServiceModel] = self.client.get_service(name)
-        if model is None:
-            raise RuntimeError(f"Service with name '{name}' was not found.")
+    def status(
+        self, name: str, *, cloud: Optional[str] = None, project: Optional[str] = None
+    ) -> ServiceStatus:
+        model = self._resolve_to_service_model(name=name, cloud=cloud, project=project)
 
         return self._service_model_to_status(model)
+
+    def wait(
+        self,
+        name: str,
+        *,
+        cloud: Optional[str] = None,
+        project: Optional[str] = None,
+        state: ServiceState,
+        timeout_s: float,
+        interval_s: float,
+    ):
+        if not isinstance(timeout_s, (int, float)):
+            raise TypeError("timeout_s must be a float")
+        if timeout_s <= 0:
+            raise ValueError("timeout_s must be >= 0")
+
+        if not isinstance(interval_s, (int, float)):
+            raise TypeError("interval_s must be a float")
+        if interval_s <= 0:
+            raise ValueError("interval_s must be >= 0")
+
+        def _get_curr_state() -> ServiceState:
+            model = self._resolve_to_service_model(
+                name=name, cloud=cloud, project=project
+            )
+
+            return self._model_state_to_state(model.current_state)
+
+        curr_state = _get_curr_state()
+        self.logger.info(
+            f"Waiting for service '{name}' to reach target state {state}, currently in state: {curr_state}"
+        )
+        for _ in self.timer.poll(timeout_s=timeout_s, interval_s=interval_s):
+            new_state = _get_curr_state()
+            if new_state != curr_state:
+                self.logger.info(
+                    f"Service '{name}' transitioned from {curr_state} to {new_state}"
+                )
+                curr_state = new_state
+
+            if curr_state == state:
+                self.logger.info(f"Service '{name}' reached target state, exiting")
+                break
+        else:
+            raise TimeoutError(
+                f"Service '{name}' did not reach target state {state} within {timeout_s}s. Last seen state: {curr_state}."
+            )
