@@ -24,7 +24,8 @@ config = ServiceConfig(
         address="http://redis-address:8888",
         certificate_path="/etc/ssl/certs/ca-certificates.crt",
     ),
-)"""
+)
+"""
 
     __doc_yaml_example__ = """\
 name: my-service
@@ -33,7 +34,8 @@ applications:
 ray_gcs_external_storage_config:
   enabled: true
   address: http://redis-address:8888
-  certificate_path: /etc/ssl/certs/ca-certificates.crt"""
+  certificate_path: /etc/ssl/certs/ca-certificates.crt
+"""
 
     enabled: bool = field(
         default=True,
@@ -69,6 +71,89 @@ ray_gcs_external_storage_config:
             raise TypeError("'certificate_path' must be a string.")
 
 
+@dataclass(frozen=True)
+class TracingConfig(ModelBase):
+    """Configuration options for tracing.
+
+    Tracing provides a way to collect telemetry on requests that are
+    handled by your Serve application. If tracing is enabled, you can instrument
+    your Serve application and export traces to a specific backend by configuring the
+    `exporter_import_path` option. This feature is currently experimental.
+    """
+
+    __doc_py_example__ = """\
+from anyscale.service.models import ServiceConfig, TracingConfig
+
+config = ServiceConfig(
+    name="my-service",
+    applications=[{"import_path": "main:app"}],
+    tracing_config=TracingConfig(
+        enabled=True,
+        exporter_import_path="my_module:custom_tracing_exporter",
+        sampling_ratio=1.0,
+    ),
+)
+"""
+
+    __doc_yaml_example__ = """\
+name: my-service
+applications:
+  - import_path: main:app
+tracing_config:
+  enabled: true
+  exporter_import_path: "my_module:custom_tracing_exporter"
+  sampling_ratio: 1.0
+"""
+
+    enabled: bool = field(
+        default=True,
+        metadata={
+            "docstring": (
+                "Flag to enable tracing. "
+                "When `True` and no `exporter_import_path` "
+                "is defined, then the default tracing exporter will be "
+                "used to export traces to serve logs. "
+                "When `False`, the other fields are ignored."
+            )
+        },
+    )
+
+    def _validate_enabled(self, enabled: bool):
+        if not isinstance(enabled, bool):
+            raise TypeError("'enabled' must be a boolean.")
+
+    exporter_import_path: Optional[str] = field(
+        default=None,
+        metadata={
+            "docstring": (
+                "Path to exporter function. Should be of the "
+                'form "module.submodule_1...submodule_n.'
+                'export_tracing". This is equivalent to '
+                '"from module.submodule_1...submodule_n import export_tracing. '
+                "Exporter function takes no arguments and returns List[SpanProcessor]."
+            )
+        },
+    )
+
+    def _validate_exporter_import_path(self, exporter_import_path: Optional[str]):
+        if exporter_import_path is not None and not isinstance(
+            exporter_import_path, str
+        ):
+            raise TypeError("'exporter_import_path' must be a string.")
+
+    sampling_ratio: float = field(
+        default=1.0,
+        metadata={
+            "docstring": "Ratio of traces to record and export. "
+            "Reducing the ratio can help with minimizing serving latency and storage overhead. "
+        },
+    )
+
+    def _validate_sampling_ratio(self, sampling_ratio: float):
+        if sampling_ratio is not None and not isinstance(sampling_ratio, float):
+            raise TypeError("'sampling_ratio' must be a float.")
+
+
 @dataclass(frozen=True)
 class ServiceConfig(WorkloadConfig):
     """Configuration options for a service."""
@@ -84,17 +169,50 @@ config = ServiceConfig(
     compute_config="my-compute-config:1",
     # A containerfile path can also be provided.
     image_uri="anyscale/image/my-image:1",
-)"""
+)
+"""
 
     __doc_yaml_example__ = """\
 name: my-service
-working_dir: .
 applications:
   - import_path: main:app
-# An inline dictionary can also be provided.
-compute_config: my-compute-config:1
-# A containerfile path can also be provided.
-image_uri: anyscale/image/my-image:1"""
+image_uri: anyscale/image/my-image:1 # (Optional) Exclusive with `containerfile`.
+containerfile: /path/to/Dockerfile # (Optional) Exclusive with `image_uri`.
+compute_config: my-compute-config:1 # (Optional) An inline dictionary can also be provided.
+working_dir: /path/to/working_dir # (Optional) If a local directory is provided, it will be uploaded automatically.
+excludes: # (Optional) List of files to exclude from being packaged up for the service.
+  - .git
+  - .env
+  - .DS_Store
+  - __pycache__
+requirements: # (Optional) List of requirements files to install. Can also be a path to a requirements.txt.
+  - emoji==1.2.0
+  - numpy==1.19.5
+env_vars: # (Optional) Dictionary of environment variables to set in the service.
+  MY_ENV_VAR: my_value
+  ANOTHER_ENV_VAR: another_value
+py_modules: # (Optional) A list of local directories or remote URIs that will be added to the Python path.
+  - /path/to/my_module
+  - s3://my_bucket/my_module
+cloud: anyscale-prod # (Optional) The name of the Anyscale Cloud.
+project: my-project # (Optional) The name of the Anyscale Project.
+query_auth_token_enabled: True # (Optional) Whether or not to gate this service behind an auth token.
+http_options:
+  request_timeout_s: 60 # (Optional) Timeout for HTTP requests in seconds. Default is no timeout.
+  keep_alive_timeout_s: 60 # (Optional) Timeout for HTTP keep-alive connections in seconds. Default is 5 seconds.
+grpc_options:
+  port: 50051 # (Optional) Port to listen on for gRPC requests. Default is 9000.
+  grpc_servicer_functions: # (Optional) List of functions to expose as gRPC servicers.
+    - my_module:my_function
+logging_config: # (Optional) Configuration options for logging.
+  encoding: JSON # JSON or TEXT.
+  log_level: INFO
+  enable_access_log: true
+tracing_config: # (Optional) Configuration options for tracing.
+  enabled: true
+  exporter_import_path: my_module:custom_tracing_exporter
+  sampling_ratio: 1.0
+"""
 
     # Override the `name` field from `WorkloadConfig` so we can document it separately for jobs and services.
     name: Optional[str] = field(
@@ -216,15 +334,15 @@ image_uri: anyscale/image/my-image:1"""
         if logging_config is not None and not isinstance(logging_config, dict):
             raise TypeError("'logging_config' must be a dict.")
 
-    ray_gcs_external_storage_config: Union[
-        RayGCSExternalStorageConfig, Dict, None
-    ] = field(
-        default=None,
-        repr=False,
-        metadata={
-            "docstring": "Configuration options for external storage for the Ray Global Control Store (GCS).",
-            "customer_hosted_only": True,
-        },
+    ray_gcs_external_storage_config: Union[RayGCSExternalStorageConfig, Dict, None] = (
+        field(
+            default=None,
+            repr=False,
+            metadata={
+                "docstring": "Configuration options for external storage for the Ray Global Control Store (GCS).",
+                "customer_hosted_only": True,
+            },
+        )
     )
 
     def _validate_ray_gcs_external_storage_config(
@@ -246,6 +364,31 @@ image_uri: anyscale/image/my-image:1"""
 
         return ray_gcs_external_storage_config
 
+    tracing_config: Union[TracingConfig, Dict, None] = field(
+        default=None,
+        repr=False,
+        metadata={
+            "docstring": "Configuration options for tracing.",
+            "customer_hosted_only": True,
+        },
+    )
+
+    def _validate_tracing_config(
+        self, tracing_config: Union[TracingConfig, Dict, None],
+    ) -> Optional[TracingConfig]:
+        if tracing_config is None:
+            return None
+
+        if isinstance(tracing_config, dict):
+            tracing_config = TracingConfig.from_dict(tracing_config)
+
+        if not isinstance(tracing_config, TracingConfig):
+            raise TypeError(
+                "'tracing_config' must be a TracingConfig or corresponding dict."
+            )
+
+        return tracing_config
+
 
 class ServiceState(ModelEnum):
     """Current state of a service."""
@@ -333,7 +476,8 @@ import anyscale
 from anyscale.service.models import ServiceStatus, ServiceVersionStatus
 
 status: ServiceStatus = anyscale.service.status("my-service")
-primary_version_status: ServiceVersionStatus = status.primary_version"""
+primary_version_status: ServiceVersionStatus = status.primary_version
+"""
 
     __doc_cli_example__ = """\
 $ anyscale service status -n my-service
@@ -343,7 +487,8 @@ state: RUNNING
 primary_version:
   id: 601bd56c4b
   state: RUNNING
-  weight: 100"""
+  weight: 100
+"""
 
     id: str = field(
         metadata={
@@ -402,7 +547,8 @@ class ServiceStatus(ModelBase):
 import anyscale
 from anyscale.service.models import ServiceStatus
 
-status: ServiceStatus = anyscale.service.status("my-service")"""
+status: ServiceStatus = anyscale.service.status("my-service")
+"""
 
     __doc_cli_example__ = """\
 $ anyscale service status -n my-service
@@ -412,7 +558,8 @@ state: RUNNING
 primary_version:
   id: 601bd56c4b
   state: RUNNING
-  weight: 100"""
+  weight: 100
+"""
 
     name: str = field(metadata={"docstring": "Unique name of the service."})
 
