@@ -2,6 +2,7 @@ from contextlib import contextmanager
 from copy import deepcopy
 import datetime
 import ipaddress
+import json
 import logging
 import os
 import random
@@ -192,7 +193,7 @@ def send_json_request(
         if resp.status_code == 500:
             raise click.ClickException(
                 "There was an internal error in this command. "
-                "Please report this to the Anyscale team at support@anyscale.zendesk.com "
+                "Please report this to the Anyscale team at support@anyscale.com "
                 "with the token '{}'.".format(resp.headers["x-trace-id"])
             )
 
@@ -362,13 +363,34 @@ def get_available_regions(boto3_session: Optional[boto3.Session] = None) -> List
 
 def get_availability_zones(
     region: str, boto3_session: Optional[boto3.Session] = None
-) -> List[str]:
+) -> Dict[str, str]:
+    """
+    Returns a mapping of availability zone ids to names for the given region.
+    """
     if boto3_session is None:
         boto3_session = boto3.Session()
     client = boto3_session.client("ec2", region_name=region)
+    return {
+        zone["ZoneId"]: zone["ZoneName"]
+        for zone in client.describe_availability_zones()["AvailabilityZones"]
+    }
+
+
+def get_memorydb_supported_zones(
+    region: str, zone_ids_to_names: Dict[str, str]
+) -> List[str]:
+    """
+    Returns a list of supported zone names for MemoryDB in the given region.
+    """
+    with open(f"{anyscale.conf.ROOT_DIR_PATH}/memorydb_supported_zones.json") as f:
+        # There's no API to get the supported zones for MemoryDB, so we need to read it from a file.
+        memorydb_supported_zones = json.load(f)
+
+    supported_zone_ids_for_cloud_region = memorydb_supported_zones.get(region, [])
+    if len(supported_zone_ids_for_cloud_region) == 0:
+        raise click.ClickException(f"MemoryDB is not supported in the region {region}.")
     return [
-        region["ZoneName"]
-        for region in client.describe_availability_zones()["AvailabilityZones"]
+        zone_ids_to_names[zone_id] for zone_id in supported_zone_ids_for_cloud_region
     ]
 
 
@@ -791,7 +813,8 @@ def prepare_cloudformation_template(
             "$ALLOWED_ORIGIN", shared_anyscale_conf.ANYSCALE_CORS_ORIGIN
         )
 
-    azs = get_availability_zones(region, boto3_session)
+    zone_ids_to_names = get_availability_zones(region, boto3_session)
+    azs = sorted(zone_ids_to_names.values())
     subnet_templates: List[str] = []
     subnets_route_table_association: List[str] = []
     subnets_with_availability_zones: List[str] = []
@@ -873,11 +896,13 @@ def prepare_cloudformation_template(
     )
 
     if enable_head_node_fault_tolerance:
+        supported_zone_names = get_memorydb_supported_zones(region, zone_ids_to_names)
+        allowed_az_indices = sorted(azs.index(az) for az in supported_zone_names)
         body = body.replace("$MEMORY_DB_OUTPUT", MEMORY_DB_OUTPUT,)
         body = body.replace(
             "$MEMORY_DB_RESOURCE",
             MEMORY_DB_RESOURCE.format(
-                "\n".join([f"        - !Ref Subnet{i}" for i in range(len(azs))])
+                "\n".join([f"        - !Ref Subnet{i}" for i in allowed_az_indices])
             ),
         )
     else:
