@@ -2,7 +2,7 @@ import json
 import re
 import secrets
 import time
-from typing import Dict, List, Optional, Set, Tuple
+from typing import Any, Dict, List, Optional, Set, Tuple
 
 from botocore.exceptions import ClientError, NoCredentialsError
 from click import ClickException
@@ -20,6 +20,8 @@ from anyscale.util import (
     generate_inline_policy_parameter,
     generate_inline_policy_resource,
     get_anyscale_cross_account_iam_policies,
+    get_availability_zones,
+    get_memorydb_supported_zones,
     MEMORY_DB_OUTPUT,
     MEMORY_DB_RESOURCE,
 )
@@ -227,7 +229,7 @@ class AWSCloudformationHandler:
             self.logger.error(f"Failed to resolve drift. Error: {e}")
             return False
 
-    def get_template_body(self):
+    def get_template_body(self) -> str:
         try:
             template_body = self.cfn_client.get_template(
                 StackName=self.aws_cloudformation_stack_id, TemplateStage="Original"
@@ -238,6 +240,17 @@ class AWSCloudformationHandler:
                 f"Failed to get template body for stack {self.aws_cloudformation_stack_id}: {e}"
             )
 
+    def get_processed_template_body(self) -> Dict[str, Any]:
+        try:
+            template_body = self.cfn_client.get_template(
+                StackName=self.aws_cloudformation_stack_id, TemplateStage="Processed"
+            )["TemplateBody"]
+            return template_body
+        except ClientError as e:
+            raise ClickException(
+                f"Failed to get template body for stack {self.aws_cloudformation_stack_id}: {e}"
+            )
+
     def update_cloudformation_stack(
         self,
         updated_template_body: str,
@@ -661,11 +674,28 @@ def try_delete_customer_drifts_policy(cloud):
             )
 
 
-def update_template_with_memorydb(template_body: str, az_num: int) -> str:
+def update_template_with_memorydb(
+    template_body: str, region: str, zone_name_to_resource_logical_id: Dict[str, str]
+) -> str:
     """
     Add MemoryDB resources and outputs to template body.
     """
-    subnet_list = [f"        - !Ref Subnet{i}" for i in range(az_num)]
+    # Get the supported zone names for MemoryDB
+    zone_ids_to_names = get_availability_zones(region)
+    supported_zone_names = get_memorydb_supported_zones(region, zone_ids_to_names)
+
+    # Get the subnet logical IDs for the supported zones
+    subnet_logical_ids = sorted(
+        zone_name_to_resource_logical_id[zone]
+        for zone in supported_zone_names
+        if zone in zone_name_to_resource_logical_id
+    )
+
+    # Generate the subnet list
+    subnet_list = [
+        f"        - !Ref {subnet_logical_id}"
+        for subnet_logical_id in subnet_logical_ids
+    ]
     memory_db_resource_str = MEMORY_DB_RESOURCE.format("\n".join(subnet_list))
 
     updated_template_body = template_body.replace(
@@ -777,7 +807,6 @@ def update_iam_role(
 def get_or_create_memorydb(
     region: str,
     aws_cloudformation_stack_id: str,
-    az_num: int,
     logger: CloudSetupLogger,
     yes: bool = False,
 ) -> str:
@@ -789,6 +818,15 @@ def get_or_create_memorydb(
     cfn_stack = cfn_handler.get_cloudformation_stack()
     cfn_stack_parameters = cfn_stack["Parameters"]
     template_body = cfn_handler.get_template_body()
+    processed_template = cfn_handler.get_processed_template_body()
+    zone_name_to_resource_logical_id = {}
+    for resource_logical_id, resource_definition in processed_template.get(
+        "Resources", []
+    ).items():
+        if resource_definition.get("Type") == "AWS::EC2::Subnet":
+            zone_name_to_resource_logical_id[
+                resource_definition["Properties"]["AvailabilityZone"]
+            ] = resource_logical_id
 
     if "AWS::MemoryDB::Cluster" in template_body:
         logger.info("MemoryDB cluster already exists in cloudformation template.")
@@ -804,7 +842,9 @@ def get_or_create_memorydb(
     )
 
     # Update template
-    updated_template_body = update_template_with_memorydb(template_body, az_num)
+    updated_template_body = update_template_with_memorydb(
+        template_body, region, zone_name_to_resource_logical_id
+    )
 
     # Update cfn stack
     cfn_handler.update_cloudformation_stack(
