@@ -1,10 +1,18 @@
+from unittest import mock
 from unittest.mock import Mock, patch
 
 from click.testing import CliRunner
 import pytest
 
-from anyscale.client.openapi_client.models import ClusterManagementStackVersions
-from anyscale.commands.cloud_commands import register_cloud, setup_cloud
+from anyscale.client.openapi_client.models import (
+    ClusterManagementStackVersions,
+    ComputeStack,
+)
+from anyscale.commands.cloud_commands import (
+    cloud_config_update,
+    register_cloud,
+    setup_cloud,
+)
 
 
 @pytest.mark.parametrize("enable_head_node_fault_tolerance", [True, False])
@@ -125,6 +133,7 @@ def test_cloud_register(
             mock_register_cloud.assert_called_once_with(
                 region=region,
                 name=name,
+                compute_stack=ComputeStack.VM,
                 project_id=project_id,
                 vpc_name=vpc_name,
                 subnet_names=subnet_names.split(","),
@@ -139,6 +148,10 @@ def test_cloud_register(
                 if enable_head_node_fault_tolerance
                 else None,
                 host_project_id=host_project_id,
+                kubernetes_namespaces=[],
+                kubernetes_ingress_external_address=None,
+                kubernetes_zones=[],
+                kubernetes_dataplane_identity=None,
                 functional_verify=None,
                 private_network=False,
                 cluster_management_stack_version=ClusterManagementStackVersions.V2,
@@ -146,3 +159,99 @@ def test_cloud_register(
                 skip_verifications=False,
                 auto_add_user=False,
             )
+
+
+@pytest.mark.parametrize("kubernetes_namespaces", [None, "namespace1"])
+@pytest.mark.parametrize("auto_add_user", [True, False])
+def test_cloud_register_generic(kubernetes_namespaces, auto_add_user):
+    runner = CliRunner()
+    mock_cloud_controller = Mock()
+    mock_register_generic_cloud = Mock()
+    mock_cloud_controller().register_generic_cloud = mock_register_generic_cloud
+
+    provider = "generic"
+    region = "us-west1"
+    name = "unit-test-generic-cloud"
+
+    args = [
+        "--provider",
+        provider,
+        "--name",
+        name,
+        "--region",
+        region,
+        "--compute-stack",
+        "k8s",
+    ]
+
+    if kubernetes_namespaces:
+        args.extend(["--kubernetes-namespaces", kubernetes_namespaces])
+
+    if auto_add_user:
+        args.append("--enable-auto-add-user")
+
+    with patch(
+        "anyscale.commands.cloud_commands.CloudController", new=mock_cloud_controller,
+    ):
+        result = runner.invoke(register_cloud, args=args, catch_exceptions=False)
+
+        if not kubernetes_namespaces:
+            assert "Please provide at least one namespace" in result.output
+            mock_register_generic_cloud.assert_not_called()
+        else:
+            mock_register_generic_cloud.assert_called_once_with(
+                name=name,
+                auto_add_user=auto_add_user,
+                region=region,
+                cloud_storage_bucket_name=None,
+                cloud_storage_bucket_endpoint=None,
+                nfs_mount_targets=[],
+                nfs_mount_path=None,
+                kubernetes_namespaces=kubernetes_namespaces.split(","),
+                kubernetes_ingress_external_address=None,
+                kubernetes_zones=[],
+            )
+
+
+@pytest.mark.parametrize(
+    ("log_ingestion_option", "response_value", "mock_click_return_value"),
+    [
+        ("--enable-log-ingestion", True, "consent"),
+        ("--enable-log-ingestion", False, "no consent"),
+        ("--disable-log-ingestion", True, True),
+        ("--disable-log-ingestion", False, False),
+    ],
+)
+@mock.patch("click.prompt")
+@mock.patch("click.confirm")
+def test_cloud_config_update_with_log_ingestion(
+    mock_click_confirm,
+    mock_click_prompt,
+    log_ingestion_option,
+    response_value,
+    mock_click_return_value,
+):
+    runner = CliRunner()
+    mock_cloud_controller = Mock()
+    mock_update_cloud_config = Mock()
+    mock_cloud_controller().update_cloud_config = mock_update_cloud_config
+    mock_click_confirm.return_value = mock_click_return_value
+    mock_click_prompt.return_value = mock_click_return_value
+
+    name = "unit-test-cloud"
+    enable_log_ingestion = "enable" in log_ingestion_option
+    args = ["--name", name, log_ingestion_option]
+
+    with patch(
+        "anyscale.commands.cloud_commands.CloudController", new=mock_cloud_controller,
+    ):
+        runner.invoke(cloud_config_update, args=args, catch_exceptions=False)
+
+        if not response_value:
+            mock_update_cloud_config.assert_not_called()
+        else:
+            mock_update_cloud_config.assert_called_once_with(
+                cloud_name=name,
+                cloud_id=None,
+                enable_log_ingestion=enable_log_ingestion,
+            )
