@@ -84,6 +84,7 @@ def cloud_test_data() -> Cloud:
         config='{"max_stopped_instances": 0}',
         state="ACTIVE",
         is_default=False,
+        customer_aggregated_logs_config_id="calc_fake_id",
     )
 
 
@@ -93,11 +94,13 @@ def project_test_data() -> Project:
         name="project_name",
         description="test project",
         cloud_id="cloud_id",
+        parent_cloud_id="cloud_id",
         initial_cluster_config="initial_config",
         id="project_id",
         created_at=datetime.now(tz=timezone.utc),
         creator_id="creator_id",
         is_owner=True,
+        is_read_only=False,
         directory_name="/directory/name",
         active_sessions=0,
         last_activity_at=datetime.now(tz=timezone.utc),
@@ -268,6 +271,7 @@ def userinfo_test_data() -> UserInfo:
         ],
         ld_hash="mock_ld_hash",
         ld_hash_fields=["email", "name", "username", "organization_ids"],
+        pylon_hash="mock_pylon_hash",
     )
 
 
