@@ -36,7 +36,6 @@ def mock_auth_api_client(base_mock_anyscale_api_client: Mock) -> Iterator[None]:
         pytest.param(None, False, True, id="happy-path-exists"),
         pytest.param(None, False, False, id="happy-path-create"),
         pytest.param(True, False, False, id="stream-removed"),
-        pytest.param(False, False, False, id="cannot-create"),
         pytest.param(True, True, False, id="timeout"),
     ],
 )
@@ -70,11 +69,6 @@ def test_get_or_create_cluster_compute(
                 ClickException("Stream removed"),
                 Mock(result=Mock(id=mock_cluster_compute_id)),
             ]
-
-    elif is_exception_stream_removed is False:
-        mock_anyscale_api_client.create_cluster_compute = Mock(
-            side_effect=ClickException("mock error")
-        )
     with patch(
         "anyscale.controllers.cloud_functional_verification_controller.CREATE_COMPUTE_CONFIG_TIMEOUT_SECONDS",
         1,
