@@ -2,7 +2,6 @@ from typing import Any, Dict, Optional, Union
 from unittest.mock import ANY, Mock, patch
 
 import click
-from click.exceptions import ClickException
 import pytest
 import yaml
 from yaml.loader import SafeLoader
@@ -14,9 +13,6 @@ from anyscale.controllers.compute_config_controller import (
 )
 from anyscale.sdk.anyscale_client.models.archive_status import ArchiveStatus
 from anyscale.sdk.anyscale_client.models.cluster_compute import ClusterCompute
-from anyscale.sdk.anyscale_client.models.cluster_computes_query import (
-    ClusterComputesQuery,
-)
 from anyscale.sdk.anyscale_client.models.clustercompute_list_response import (
     ClustercomputeListResponse,
 )
@@ -30,7 +26,6 @@ from anyscale.sdk.anyscale_client.models.compute_template_query import (
 from anyscale.sdk.anyscale_client.models.create_cluster_compute import (
     CreateClusterCompute,
 )
-from anyscale.sdk.anyscale_client.models.text_query import TextQuery
 from anyscale.utils.entity_arg_utils import EntityType, IdBasedEntity, NameBasedEntity
 
 
@@ -121,7 +116,6 @@ def mock_sdk_client(cluster_compute_test_data: ClusterCompute) -> Mock:
     mock_sdk_client.create_cluster_compute = Mock(
         return_value=ClustercomputeResponse(result=cluster_compute_test_data)
     )
-    mock_sdk_client.delete_cluster_compute = Mock()
     mock_sdk_client.search_cluster_computes = Mock(
         return_value=ClustercomputeListResponse(results=[cluster_compute_test_data])
     )
@@ -178,42 +172,6 @@ def test_create(
     )
 
 
-@pytest.mark.parametrize("cluster_compute_name", [None, "test_name"])
-@pytest.mark.parametrize("id", [None, "cpt_123"])
-def test_delete(
-    mock_auth_api_client,
-    cluster_compute_name: Optional[str],
-    id: Optional[str],  # noqa: A002
-    cluster_compute_test_data: ClusterCompute,
-) -> None:
-    compute_config_controller = ComputeConfigController()
-
-    if cluster_compute_name is None and id is None:
-        with pytest.raises(ClickException):
-            compute_config_controller.delete(cluster_compute_name, id)
-        compute_config_controller.anyscale_api_client.delete_cluster_compute.assert_not_called()
-    elif cluster_compute_name is not None and id is not None:
-        with pytest.raises(ClickException):
-            compute_config_controller.delete(cluster_compute_name, id)
-        compute_config_controller.anyscale_api_client.delete_cluster_compute.assert_not_called()
-    elif cluster_compute_name and id is None:
-        compute_config_controller.delete(cluster_compute_name, id)
-        compute_config_controller.anyscale_api_client.search_cluster_computes.assert_called_once_with(
-            ClusterComputesQuery(name=TextQuery(equals=cluster_compute_name))
-        )
-        compute_config_controller.anyscale_api_client.delete_cluster_compute.assert_called_once_with(
-            cluster_compute_test_data.id
-        )
-    elif cluster_compute_name is None and id:
-        compute_config_controller.delete(cluster_compute_name, id)
-        compute_config_controller.anyscale_api_client.get_cluster_compute.assert_called_once_with(
-            id
-        )
-        compute_config_controller.anyscale_api_client.delete_cluster_compute.assert_called_once_with(
-            id
-        )
-
-
 @pytest.mark.parametrize(
     "cluster_compute_entity",
     [IdBasedEntity(id="cpt_123"), NameBasedEntity(name="compute_config_name"),],
