@@ -2,6 +2,7 @@ import os
 from typing import Optional
 from unittest.mock import ANY, Mock, mock_open as mock_open_factory, patch
 
+import click
 import pytest
 
 from anyscale.client.openapi_client import Project, UserInfo
@@ -282,13 +283,27 @@ def test_list(
 
 
 @pytest.mark.parametrize("existing_projects", [[], [Mock()]])
-def test_create_project(mock_auth_api_client, existing_projects) -> None:
+@pytest.mark.parametrize("parent_cloud_id", ["mock_parent_cloud_id", None])
+@pytest.mark.parametrize("cloud_isolation_ff_on", [True, False])
+def test_create_project(
+    mock_auth_api_client,
+    existing_projects,
+    parent_cloud_id: Optional[str],
+    cloud_isolation_ff_on: bool,
+) -> None:
     project_controller = ProjectController()
     project_controller.api_client.find_project_by_project_name_api_v2_projects_find_by_name_get = Mock(
         return_value=Mock(results=existing_projects)
     )
+    project_controller.api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get = Mock(
+        return_value=Mock(result=Mock(is_on=cloud_isolation_ff_on))
+    )
 
-    project_controller.create("proj_name")
+    if cloud_isolation_ff_on and not parent_cloud_id and not len(existing_projects):
+        with pytest.raises(click.ClickException):
+            project_controller.create("proj_name", parent_cloud_id)
+        return
+    project_controller.create("proj_name", parent_cloud_id)
 
     if existing_projects:
         project_controller.api_client.create_project_api_v2_projects_post.assert_not_called()
@@ -298,5 +313,6 @@ def test_create_project(mock_auth_api_client, existing_projects) -> None:
                 "name": "proj_name",
                 "description": "",
                 "initial_cluster_config": None,
+                "parent_cloud_id": parent_cloud_id,
             }
         )
