@@ -32,7 +32,7 @@ RAY_VERSION = "1.6.0"
 RAY_COMMIT = "7916500c43de46721c51e6b95fb51cfa2c6078ba"
 
 
-def _make_session(i: int, state: str, cloud_id: str = None) -> Session:
+def _make_session(i: int, state: str, cloud_id: Optional[str] = None) -> Session:
     sess = Session(
         id="session_id",
         name=f"cluster-{i}",
@@ -90,44 +90,39 @@ def _make_build() -> Build:
 def _make_compute_template_config() -> ComputeTemplateConfig:
 
     return ComputeTemplateConfig(  # noqa: PIE804
-        **{
-            "cloud_id": "fake-cloud-id",
-            "region": "fake-region",
-            "allowed_azs": ["fake-az1"],
-            "head_node_type": {
-                "name": "head-node-name",
-                "instance_type": "fake-head-instance-type",
-                "resources": {
-                    "cpu": 3,
-                    "object_store_memory": 5,
-                    "custom_resources": {
-                        "custom_resource_1": 10,
-                        "custom_resource_2": 12,
-                    },
-                },
+        cloud_id="fake-cloud-id",
+        region="fake-region",
+        allowed_azs=["fake-az1"],
+        head_node_type={
+            "name": "head-node-name",
+            "instance_type": "fake-head-instance-type",
+            "resources": {
+                "cpu": 3,
+                "object_store_memory": 5,
+                "custom_resources": {"custom_resource_1": 10, "custom_resource_2": 12,},
             },
-            "worker_node_types": [
+        },
+        worker_node_types=[
+            {
+                "name": "worker-node-name",
+                "instance_type": "fake-worker-instance-type",
+                "min_workers": 0,
+                "max_workers": 10,
+                "use_spot": True,
+                "resources": {"cpu": 7, "memory": 11},
+            }
+        ],
+        aws={
+            "SubnetId": "fake-subnet-id",
+            "SecurityGroupIds": ["fake-security-group-id"],
+            "IamInstanceProfile": {"Arn": "fake-iam-arn"},
+            "TagSpecifications": [
                 {
-                    "name": "worker-node-name",
-                    "instance_type": "fake-worker-instance-type",
-                    "min_workers": 0,
-                    "max_workers": 10,
-                    "use_spot": True,
-                    "resources": {"cpu": 7, "memory": 11},
-                }
+                    "ResourceType": "instance",
+                    "Tags": [{"Key": "fake-key", "Value": "fake-value"}],
+                },
             ],
-            "aws": {
-                "SubnetId": "fake-subnet-id",
-                "SecurityGroupIds": ["fake-security-group-id"],
-                "IamInstanceProfile": {"Arn": "fake-iam-arn"},
-                "TagSpecifications": [
-                    {
-                        "ResourceType": "instance",
-                        "Tags": [{"Key": "fake-key", "Value": "fake-value"}],
-                    },
-                ],
-            },
-        }
+        },
     )
 
 
@@ -196,7 +191,7 @@ def _make_test_builder(
     session_states: Optional[List[str]] = None,
     setup_project_dir: bool = True,
     create_build: bool = True,
-    cloud_id: str = None,
+    cloud_id: Optional[str] = None,
 ) -> Tuple[Any, Any, Any, Any]:
     if session_states is None:
         session_states = ["Running"]
@@ -611,7 +606,6 @@ def test_new_proj_connect_params(
         cloud_name=None,
         cluster_compute_dict=None,
         cluster_compute_name=None,
-        cluster_compute_version=None,
     )
     mock_get_prepare_cluster_block.assert_called_once_with(
         allow_public_internet_traffic=builder._allow_public_internet_traffic,
@@ -630,7 +624,6 @@ def test_new_proj_connect_params(
         project_id=mock_project_block.project_id,
         ray=builder._ray,
         log_output=True,
-        cluster_compute_version=builder._cluster_compute_version,
     )
     mock_get_start_interactive_session_block.assert_called_once_with(
         cluster=mock_cluster,
@@ -718,7 +711,6 @@ def test_connect_with_cloud(
         project_id=mock_get_project_block.return_value.project_id,
         ray=builder._ray,
         log_output=True,
-        cluster_compute_version=builder._cluster_compute_version,
     )
 
 
@@ -761,7 +753,6 @@ def test_connect_with_cluster_env_dict(
         project_id=mock_get_project_block.return_value.project_id,
         ray=builder._ray,
         log_output=True,
-        cluster_compute_version=builder._cluster_compute_version,
     )
 
 
@@ -981,11 +972,10 @@ def test_pin_protobuf_in_runtime_env_if_needed(
             with file.open("w") as f:
                 yaml.dump(conda_dict, f)
             runtime_env = {"conda": str(file)}
-    else:
-        if field == "pip":
-            runtime_env = {"pip": ["requests"]}
-        elif field == "conda":
-            runtime_env = {"conda": {"dependencies": ["pip", {"pip": ["requests"]}]}}
+    elif field == "pip":
+        runtime_env = {"pip": ["requests"]}
+    elif field == "conda":
+        runtime_env = {"conda": {"dependencies": ["pip", {"pip": ["requests"]}]}}
 
     sdk = Mock()
     _api_client = Mock()
@@ -1117,7 +1107,6 @@ def test_set_runtime_env_in_job_config(
         project_id=mock_get_project_block.return_value.project_id,
         ray=connect_instance._ray,
         log_output=True,
-        cluster_compute_version=connect_instance._cluster_compute_version,
     )
     mock_get_start_interactive_session_block.assert_called_once_with(
         cluster=mock_cluster,
