@@ -17,6 +17,7 @@ from anyscale.project import (
     get_parent_cloud_id_and_name_of_project,
     get_proj_id_from_name,
     get_proj_name_from_id,
+    get_project_id_for_cloud_from_name,
     get_project_session,
     get_project_sessions,
     infer_project_id,
@@ -99,6 +100,31 @@ def test_get_proj_id_from_name(project_test_data: Project, owner: str) -> None:
     )
 
 
+def test_get_project_id_for_cloud_from_name(project_test_data: Project) -> None:
+    mock_api_client = Mock()
+    mock_api_client.find_project_by_project_name_api_v2_projects_find_by_name_get.return_value.results = [
+        project_test_data
+    ]
+    project_id = get_project_id_for_cloud_from_name(
+        project_test_data.name, project_test_data.cloud_id, mock_api_client
+    )
+
+    assert project_id == project_test_data.id
+    mock_api_client.find_project_by_project_name_api_v2_projects_find_by_name_get.assert_called_once_with(
+        project_test_data.name
+    )
+
+    default_project_id = "default_project_id"
+    mock_get_default_project = Mock(return_value=Mock(id="default_project_id"))
+
+    with patch("anyscale.project.get_default_project", mock_get_default_project):
+        project_id = get_project_id_for_cloud_from_name(
+            "default", project_test_data.cloud_id, mock_api_client
+        )
+    assert project_id == default_project_id
+    mock_get_default_project.assert_called_once()
+
+
 def test_create_new_proj_def(project_test_data: Project) -> None:
     mock_api_client = Mock()
 
