@@ -79,3 +79,9 @@ class TestLogger:
     def error(self, msg: str):
         self._error_messages.append(msg)
         self._delegate.error(msg)
+
+
+def verify_file_content(file_path: str, expected_content: str):
+    with open(file_path, "r") as file:
+        content = file.read()
+        assert content == expected_content
