@@ -1,25 +1,76 @@
 from dataclasses import dataclass, field
+import os
 import re
 from tempfile import TemporaryDirectory
-from typing import Dict, List, Optional, Union
+import traceback
+from typing import Dict, List, Optional, Tuple, Union
 
 import click
 from click.testing import CliRunner
 import pytest
 
-from anyscale._private.docgen.__main__ import generate
+from anyscale._private.docgen.__main__ import ALL_MODULES, generate
 from anyscale._private.docgen.generator import MarkdownGenerator, Module
+from anyscale._private.docgen.generator_legacy import parse_legacy_sdks
 from anyscale._private.models import ModelBase, ModelEnum
 from anyscale._private.sdk import sdk_command
+from anyscale.commands.util import AnyscaleCommand, LegacyAnyscaleCommand
 
 
 def test_full_smoke_test():
     runner = CliRunner()
     with TemporaryDirectory() as d:
         result = runner.invoke(generate, [d])
+        # should have an output per module
+        expected_output_count = len(ALL_MODULES)
 
-        assert result.exit_code == 0
-        assert result.output.count("Writing output file") == 4
+        if result.exception:
+            e = result.exception
+            stacktrace = traceback.format_exception(type(e), e, e.__traceback__)
+            formatted_exception = "".join(stacktrace)
+
+        assert result.exit_code == 0, result.output or formatted_exception
+        assert result.output.count("Writing output file") == expected_output_count
+
+
+def test_parse_legacy_sdks():
+    _TEST_API_MD = """\
+some random text here
+
+### fake_legacy_sdk
+
+this is a fake legacy [sdk](./models.md#fakesdk)
+"""
+
+    _TEST_MODEL_MD = """\
+this is random
+
+## FakeLegacyModel
+
+this is a fake legacy [model](#fakemodel)
+
+# next section
+"""
+
+    with TemporaryDirectory() as tmp:
+        with open(os.path.join(tmp, "api.md"), "w") as f:
+            f.write(_TEST_API_MD)
+
+        with open(os.path.join(tmp, "model.md"), "w") as f:
+            f.write(_TEST_MODEL_MD)
+
+        legacy_sdks, legacy_models = parse_legacy_sdks(
+            os.path.join(tmp, "api.md"), os.path.join(tmp, "model.md"),
+        )
+        assert legacy_sdks[0].name == "fake_legacy_sdk"
+        assert (
+            legacy_sdks[0].docstring == "this is a fake legacy [sdk](#fakesdk-legacy)"
+        )
+        assert legacy_models[0].name == "FakeLegacyModel"
+        assert (
+            legacy_models[0].docstring
+            == "this is a fake legacy [model](#fakemodel-legacy)"
+        )
 
 
 def _fake_module(
@@ -28,6 +79,8 @@ def _fake_module(
     models: Optional[List] = None,
     cli_commands: Optional[List] = None,
     sdk_commands: Optional[List] = None,
+    cli_command_group_prefix: Optional[Dict[click.Command, str]] = None,
+    legacy_cli_commands: Optional[List] = None,
 ) -> Module:
     return Module(
         title=name,
@@ -37,6 +90,9 @@ def _fake_module(
         sdk_prefix=f"anyscale.{name.lower()}",
         sdk_commands=sdk_commands or [],
         models=models or [],
+        cli_command_group_prefix=cli_command_group_prefix,
+        legacy_cli_prefix=f"anyscale legacy {name.lower()}",
+        legacy_cli_commands=legacy_cli_commands or [],
     )
 
 
@@ -61,7 +117,7 @@ run_python_command()
         output = MarkdownGenerator(
             modules=[_fake_module(models=[FakeConfig])]
         ).generate()
-        assert len(output) == 2
+        assert len(output) == 1
         assert "## `FakeConfig`" in output["fake.md"]
         assert "FakeConfig does really special things." in output["fake.md"]
         assert "**`foo` (str)**: Foo docstring." in output["fake.md"]
@@ -194,11 +250,11 @@ run_python_command()
         assert "**`foo` (str)**: foo docs" in output["fake.md"]
         assert "**`bar` (str | None)**: bar docs" in output["fake.md"]
         assert (
-            "**`baz` (Dict[str, [FakeStatus](fake#fakestatus)])**: baz docs"
+            "**`baz` (Dict[str, [FakeStatus](fake.md#fakestatus)])**: baz docs"
             in output["fake.md"]
         )
         assert (
-            "**`qux` (List[[FakeEnum](fake#fakeenum) | None])**: qux docs"
+            "**`qux` (List[[FakeEnum](fake.md#fakeenum) | None])**: qux docs"
             in output["fake.md"]
         )
 
@@ -262,7 +318,7 @@ class TestGenerateModelEnumType:
             }
 
         output = MarkdownGenerator(modules=[_fake_module(models=[FakeEnum])]).generate()
-        assert len(output) == 2
+        assert len(output) == 1
         assert "FakeEnum is highly important." in output["fake.md"]
         assert "**`FOO`**: Foo is quite important." in output["fake.md"]
         assert "**`BAR`**: But bar is even more so." in output["fake.md"]
@@ -303,7 +359,7 @@ class TestGenerateModelEnumType:
 
 class TestGenerateCLICommand:
     def test_basic(self):
-        @click.command("do-something")
+        @click.command("do-something", cls=AnyscaleCommand, example="infinite loop")
         @click.argument("the-arg")
         @click.option("-m", "--maybe", help="Maybe do something else.")
         def do_something():
@@ -318,6 +374,104 @@ class TestGenerateCLICommand:
         assert "**`-m/--maybe`**: Maybe do something else." in output["fake.md"]
         assert "--help" not in output["fake.md"]
 
+    def test_secondary_opts(self):
+        @click.command("do-something", cls=AnyscaleCommand, example="infinite loop")
+        @click.argument("the-arg")
+        @click.option("--enable/--disable", help="Enable or disable something.")
+        def do_something():
+            """This is the description of what to do."""
+
+        output = MarkdownGenerator(
+            modules=[_fake_module(cli_commands=[do_something])]
+        ).generate()
+        assert "## `anyscale fake do-something`" in output["fake.md"]
+        assert "anyscale fake do-something [OPTIONS] THE_ARG" in output["fake.md"]
+        assert "This is the description of what to do." in output["fake.md"]
+        assert (
+            "**`--enable/--disable`**: Enable or disable something."
+            in output["fake.md"]
+        )
+        assert "--help" not in output["fake.md"]
+
+    def test_command_with_group(self):
+        @click.group("my_group", help="This is my group.")
+        def my_group() -> None:
+            pass
+
+        @my_group.command("do-something", cls=AnyscaleCommand, example="infinite loop")
+        def do_something():
+            """This is the description of what to do."""
+
+        output = MarkdownGenerator(
+            modules=[
+                _fake_module(
+                    cli_commands=[do_something],
+                    cli_command_group_prefix={do_something: "my_group"},
+                ),
+            ]
+        ).generate()
+        assert "## `anyscale fake my_group do-something`" in output["fake.md"]
+        assert "anyscale fake my_group do-something [OPTIONS]" in output["fake.md"]
+        assert "--help" not in output["fake.md"]
+        assert '<TabItem value="cli" label="CLI">' in output["fake.md"]
+        assert "infinite loop" in output["fake.md"]
+
+    def test_legacy_command_with_group(self):
+        @click.group("my_group", help="This is my group.")
+        def my_group() -> None:
+            pass
+
+        @my_group.command("new", cls=AnyscaleCommand, example="infinite loop")
+        def new():
+            """This is the description of what to do."""
+
+        @my_group.command(
+            "old", cls=LegacyAnyscaleCommand, new_cli=new, new_prefix="anyscale"
+        )
+        def old():
+            """This is the description of what to do."""
+
+        @my_group.command("limited", cls=LegacyAnyscaleCommand, is_limited_support=True)
+        def limited():
+            """This is the description of what to do."""
+
+        output = MarkdownGenerator(
+            modules=[_fake_module(legacy_cli_commands=[old, limited],),]
+        ).generate()
+        assert "### `anyscale legacy fake old`" in output["fake.md"]
+        assert '<span class="label-h3 label-legacy">Legacy</span>' in output["fake.md"]
+        assert (
+            "This command is deprecated. Upgrade to [anyscale new](#anyscale-new)."
+            in output["fake.md"]
+        )
+        assert "### `anyscale legacy fake limited`" in output["fake.md"]
+        assert '<span class="label-h3 label-legacy">Legacy</span>' in output["fake.md"]
+        assert (
+            "This command is not actively maintained. Use with caution."
+            in output["fake.md"]
+        )
+
+    def test_beta_command_with_group(self):
+        @click.group("my_group", help="This is my group.")
+        def my_group() -> None:
+            pass
+
+        @my_group.command(
+            "new", cls=AnyscaleCommand, example="infinite loop", is_beta=True
+        )
+        def new():
+            """This is the description of what to do."""
+
+        output = MarkdownGenerator(
+            modules=[_fake_module(cli_commands=[new])]
+        ).generate()
+        assert "### `anyscale fake new`" in output["fake.md"]
+        assert '<span class="label-h3 label-beta">Beta</span>' in output["fake.md"]
+        assert (
+            "This command undergoes rapid iteration. Users must be tolerant of change."
+            in output["fake.md"]
+        )
+
 
 class TestGenerateSDKCommand:
     def test_basic(self):
@@ -432,6 +586,7 @@ class TestGenerateSDKCommand:
                 "baz": "baz docs",
                 "qux": "qux docs",
                 "quux": "quux docs",
+                "quuz": "quuz docs",
             },
         )
         def do_something(
@@ -440,6 +595,7 @@ class TestGenerateSDKCommand:
             baz: Union[Optional[FakeConfig], FakeStatus],
             qux: List[FakeEnum],
             quux: Dict[str, Optional[FakeEnum]],
+            quuz: Tuple[str, int],
         ) -> Optional[FakeStatus]:
             """Might do something."""
 
@@ -452,78 +608,24 @@ class TestGenerateSDKCommand:
             ]
         ).generate()
 
-        assert "**`foo` ([FakeEnum](fake#fakeenum))**: foo docs" in output["fake.md"]
+        assert "**`foo` ([FakeEnum](fake.md#fakeenum))**: foo docs" in output["fake.md"]
         assert (
-            "**`bar` ([FakeStatus](fake#fakestatus) | None)**: bar docs"
+            "**`bar` ([FakeStatus](fake.md#fakestatus) | None)**: bar docs"
             in output["fake.md"]
         )
         assert (
-            "**`baz` ([FakeConfig](fake#fakeconfig) | None | [FakeStatus](fake#fakestatus))**: baz docs"
+            "**`baz` ([FakeConfig](fake.md#fakeconfig) | None | [FakeStatus](fake.md#fakestatus))**: baz docs"
             in output["fake.md"]
         )
         assert (
-            "**`qux` (List[[FakeEnum](fake#fakeenum)])**: qux docs" in output["fake.md"]
-        )
-        assert (
-            "**`quux` (Dict[str, [FakeEnum](fake#fakeenum) | None])**: quux docs"
+            "**`qux` (List[[FakeEnum](fake.md#fakeenum)])**: qux docs"
             in output["fake.md"]
         )
-        assert "**Returns**: [FakeStatus](fake#fakestatus) | None" in output["fake.md"]
-
-
-class TestIndex:
-    def test_basic(self):
-        @click.command("do-something")
-        @click.argument("the-arg")
-        @click.option("-m", "--maybe", help="Maybe do something else.")
-        def do_something_cli():
-            """This is the description of what to do."""
-
-        @dataclass(frozen=True)
-        class FakeConfig(ModelBase):
-            """FakeConfig does really special things."""
-
-            __doc_py_example__ = "FakeConfig()"
-            __doc_yaml_example__ = "fake: config"
-
-        @sdk_command(
-            "TEST_KEY",
-            None,
-            doc_py_example="do_something()",
-            arg_docstrings={"foo": "foo docs"},
-        )
-        def do_something_sdk(foo: str):
-            """Might do something."""
-
-        output = MarkdownGenerator(
-            modules=[
-                _fake_module(
-                    cli_commands=[do_something_cli],
-                    sdk_commands=[do_something_sdk],
-                    models=[FakeConfig],
-                ),
-                _fake_module(
-                    name="Other",
-                    cli_commands=[do_something_cli],
-                    sdk_commands=[do_something_sdk],
-                    models=[FakeConfig],
-                ),
-            ]
-        ).generate()
-        assert set(output.keys()) == {"index.md", "fake.md", "other.md"}
-
         assert (
-            output["index.md"]
-            == """\
-# Platform API
-
-- [Fake API Reference](fake)
-  - [Fake Models](fake#fake-models)
-  - [Fake CLI](fake#fake-cli)
-  - [Fake SDK](fake#fake-sdk)
-- [Other API Reference](other)
-  - [Other Models](other#other-models)
-  - [Other CLI](other#other-cli)
-  - [Other SDK](other#other-sdk)
-"""
+            "**`quux` (Dict[str, [FakeEnum](fake.md#fakeenum) | None])**: quux docs"
+            in output["fake.md"]
+        )
+        assert "**`quuz` (Tuple[str, int])**: quuz docs" in output["fake.md"]
+        assert (
+            "**Returns**: [FakeStatus](fake.md#fakestatus) | None" in output["fake.md"]
         )
