@@ -1,4 +1,3 @@
-import copy
 import os
 from typing import Any, Dict, Generator, Optional
 
@@ -6,8 +5,9 @@ import click
 from click.testing import CliRunner
 import pytest
 
+from anyscale._private.models.image_uri import ANYSCALE_CLUSTER_ENV_PREFIX
 from anyscale._private.sdk import _LAZY_SDK_SINGLETONS
-from anyscale.commands.service_commands import deploy, status
+from anyscale.commands.service_commands import deploy, status, wait
 from anyscale.compute_config.models import ComputeConfig, HeadNodeConfig
 from anyscale.service.commands import _SERVICE_SDK_SINGLETON_KEY
 from anyscale.service.models import (
@@ -37,29 +37,22 @@ class FakeServiceSDK:
     DEFAULT_SERVICE_NAME = "default-fake-service-name"
 
     def __init__(self):
-        self._deployed_config: Optional[ServiceConfig] = None
-        self._deployed_kwargs: Dict[str, Any] = {}
-        self._fetched_name: Optional[str] = None
-
-    @property
-    def deployed_config(self) -> Optional[ServiceConfig]:
-        return self._deployed_config
-
-    @property
-    def deployed_kwargs(self) -> Dict[str, Any]:
-        return copy.deepcopy(self._deployed_kwargs)
+        self.deployed_config: Optional[ServiceConfig] = None
+        self.deployed_kwargs: Dict[str, Any] = {}
+        self.fetched_name: Optional[str] = None
+        self.waited_name: Optional[str] = None
+        self.waited_state: Optional[ServiceState] = None
+        self.waited_timeout_s: Optional[float] = None
 
     def deploy(self, config: ServiceConfig, **kwargs):
         assert isinstance(config, ServiceConfig)
-        self._deployed_config = config
-        self._deployed_kwargs = kwargs
+        self.deployed_config = config
+        self.deployed_kwargs = kwargs
 
-    @property
-    def fetched_name(self) -> Optional[str]:
-        return self._fetched_name
-
-    def status(self, name: Optional[str] = None) -> ServiceStatus:
-        self._fetched_name = name
+    def status(
+        self, name: str, *, cloud: Optional[str] = None, project: Optional[str] = None
+    ) -> ServiceStatus:
+        self.fetched_name = name
         return ServiceStatus(
             id=self.DEFAULT_SERVICE_ID,
             name=name or self.DEFAULT_SERVICE_NAME,
@@ -80,6 +73,20 @@ class FakeServiceSDK:
             ),
         )
 
+    def wait(
+        self,
+        name: str,
+        *,
+        cloud: Optional[str] = None,
+        project: Optional[str] = None,
+        state: ServiceState,
+        timeout_s: float,
+        interval_s: float,
+    ):
+        self.waited_name = name
+        self.waited_state = state
+        self.waited_timeout_s = timeout_s
+
 
 @pytest.fixture()
 def fake_service_sdk() -> Generator[FakeServiceSDK, None, None]:
@@ -274,6 +281,34 @@ class TestDeploy:
             message="Only one of '--containerfile' and '--image-uri' can be provided.",
         )
 
+    def test_deploy_registry_login_secret_without_custom_image_uri(self,):
+        runner = CliRunner()
+
+        # No image uri provided
+        result = runner.invoke(
+            deploy, ["main:app", "--registry-login-secret", "secret",],
+        )
+        _assert_error_message(
+            result,
+            message="Registry login secret can only be used with an image that is not hosted on Anyscale.",
+        )
+
+        # Anyscale image uri provided
+        result = runner.invoke(
+            deploy,
+            [
+                "main:app",
+                "--registry-login-secret",
+                "secret",
+                "--image-uri",
+                f"{ANYSCALE_CLUSTER_ENV_PREFIX}some-image",
+            ],
+        )
+        _assert_error_message(
+            result,
+            message="Registry login secret can only be used with an image that is not hosted on Anyscale.",
+        )
+
     def test_deploy_from_file_override_options(self, fake_service_sdk):
         runner = CliRunner()
 
@@ -331,6 +366,8 @@ class TestDeploy:
                 FULL_CONFIG_PATH,
                 "--image-uri",
                 "docker.io/user/override-image:latest",
+                "--registry-login-secret",
+                "secret-name",
             ],
         )
         assert result.exit_code == 0
@@ -340,6 +377,7 @@ class TestDeploy:
             applications=[{"import_path": "main:app"}],
             name="test-name-from-file",
             image_uri="docker.io/user/override-image:latest",
+            registry_login_secret="secret-name",
             compute_config=inline_compute_config,
             working_dir="test-working-dir",
             excludes=["test"],
@@ -456,8 +494,10 @@ class TestStatus:
         # Test passing no name (would be picked up as workspace name).
         runner = CliRunner()
         result = runner.invoke(status)
-        assert result.exit_code == 0
-        assert fake_service_sdk.fetched_name is None
+        _assert_error_message(
+            result,
+            message="Service name must be provided using '--name' or in a config file using '-f'",
+        )
 
     def test_name_and_config_file(self, fake_service_sdk):
         runner = CliRunner()
@@ -532,3 +572,75 @@ class TestStatus:
         assert "query_auth_token" in result.stdout
         assert "config" in result.stdout
         assert "applications" in result.stdout
+
+
+class TestWait:
+    def test_no_name(self, fake_service_sdk):
+        # Test passing no name (would be picked up as workspace name).
+        runner = CliRunner()
+        result = runner.invoke(wait)
+        _assert_error_message(
+            result,
+            message="Service name must be provided using '--name' or in a config file using '-f'",
+        )
+
+    def test_name(self, fake_service_sdk):
+        runner = CliRunner()
+        result = runner.invoke(wait, ["-n", "custom-name"])
+        assert result.exit_code == 0
+        assert fake_service_sdk.waited_name == "custom-name"
+
+    def test_service_config_file_not_found(self, fake_service_sdk):
+        runner = CliRunner()
+        result = runner.invoke(wait, ["-f", "nonexistent.yaml"])
+        _assert_error_message(
+            result, message="Config file not found at path: 'nonexistent.yaml'."
+        )
+
+    def test_empty_config_file(self, fake_service_sdk):
+        runner = CliRunner()
+        result = runner.invoke(wait, ["-f", EMPTY_CONFIG_PATH])
+        _assert_error_message(
+            result,
+            message=f"No 'name' property found in config file '{EMPTY_CONFIG_PATH}'.",
+        )
+
+    def test_service_config_file_missing_name(self, fake_service_sdk):
+        runner = CliRunner()
+        result = runner.invoke(wait, ["-f", MINIMAL_CONFIG_PATH])
+        _assert_error_message(
+            result,
+            message=f"No 'name' property found in config file '{MINIMAL_CONFIG_PATH}'.",
+        )
+
+    def test_service_config_file(self, fake_service_sdk):
+        runner = CliRunner()
+        result = runner.invoke(wait, ["-f", FULL_CONFIG_PATH])
+        assert result.exit_code == 0
+        assert fake_service_sdk.waited_name == "test-name-from-file"
+
+    def test_service_config_file_only_reads_name(self, fake_service_sdk):
+        # `wait` CLI path should ignore any other fields in the config file aside from `name`.
+        runner = CliRunner()
+        result = runner.invoke(wait, ["-f", NAME_AND_GIBBERISH_CONFIG_PATH])
+        assert result.exit_code == 0
+        assert fake_service_sdk.waited_name == "service-name-from-file"
+
+    def test_state(self, fake_service_sdk):
+        runner = CliRunner()
+        result = runner.invoke(wait, ["-n", "custom-name", "--state", "TERMINATED"])
+        assert result.exit_code == 0
+        assert fake_service_sdk.waited_name == "custom-name"
+        assert fake_service_sdk.waited_state == ServiceState.TERMINATED
+
+    def test_bad_state(self, fake_service_sdk):
+        runner = CliRunner()
+        result = runner.invoke(wait, ["-n", "custom-name", "--state", "FOOBAR"])
+        assert result.exit_code == 1
+        assert "'FOOBAR' is not a valid ServiceState" in str(result.stdout)
+
+    def test_timeout(self, fake_service_sdk):
+        runner = CliRunner()
+        result = runner.invoke(wait, ["-n", "custom-name", "--timeout-s=30"])
+        assert result.exit_code == 0
+        assert fake_service_sdk.waited_timeout_s == 30
