@@ -13,6 +13,7 @@ from anyscale.service.models import (
     ServiceStatus,
     ServiceVersionState,
     ServiceVersionStatus,
+    TracingConfig,
 )
 
 
@@ -229,6 +230,25 @@ class TestServiceConfig:
                 ray_gcs_external_storage_config=["address"],
             )
 
+    def test_tracing_config(self):
+        config = ServiceConfig(applications=[{"import_path": "main:app"}])
+        assert config.tracing_config is None
+
+        config = ServiceConfig(
+            applications=[{"import_path": "main:app"}],
+            tracing_config=TracingConfig(exporter_import_path="path:exporter",),
+        )
+        assert config.tracing_config == TracingConfig(
+            exporter_import_path="path:exporter"
+        )
+
+        with pytest.raises(
+            TypeError, match="'tracing_config' must be a TracingConfig.",
+        ):
+            ServiceConfig(
+                applications=[{"import_path": "main:app"}], tracing_config=["path"],
+            )
+
     def test_options(self):
         config = ServiceConfig(applications=[{"import_path": "main:app"}])
 
