@@ -68,6 +68,26 @@ class TestWorkloadConfig:
         # Test setting fields all at once.
         assert config.options(**options) == WorkloadConfig(**options)
 
+    def test_cloud(self):
+        config = WorkloadConfig()
+        assert config.cloud is None
+
+        config = WorkloadConfig(cloud="my-custom-cloud")
+        assert config.cloud == "my-custom-cloud"
+
+        with pytest.raises(TypeError, match="'cloud' must be a string"):
+            WorkloadConfig(cloud=123)
+
+    def test_project(self):
+        config = WorkloadConfig()
+        assert config.project is None
+
+        config = WorkloadConfig(project="my-custom-project")
+        assert config.project == "my-custom-project"
+
+        with pytest.raises(TypeError, match="'project' must be a string"):
+            WorkloadConfig(project=123)
+
     def test_invalid_requirements(self):
         WorkloadConfig(requirements="test")
         WorkloadConfig(requirements=["test"])
